CDECK  ID>, HWH2T1.
*CMZ :-        -27/02/01  17:04:16  by  Peter Richardson
*-- Author :    Peter Richardson
C-----------------------------------------------------------------------
      FUNCTION HWH2T1(I,J,K,L,Z1,Z2,P1)
C-----------------------------------------------------------------------
C     Returns the amplitude T1 from Nucl. Phys. B262 (1985) 235-262
C     I-L are the particles
C     Z1 and Z2 are the decay products of the Z
C     P1 is the polarization of the line I,J
C-----------------------------------------------------------------------
      INCLUDE 'HERWIG65.INC'
      DOUBLE COMPLEX HWH2T1,S,D
      INTEGER I,J,K,L,Z1,Z2,P1
      COMMON/HWHEWS/S(8,8,2),D(8,8)
      IF(P1.EQ.1) THEN
        HWH2T1 = TWO*S(I,Z2,1)*S(Z1,J,2)
      ELSEIF(P1.EQ.2) THEN
        HWH2T1 = TWO*S(I,Z1,2)*S(Z2,J,1)
      ELSE
        CALL HWWARN('HWH2T1',500,*999)
      ENDIF
 999  END
CDECK  ID>, HWH2T2
*CMZ :-        -27/02/01  17:04:16  by  Peter Richardson
*-- Author :    Peter Richardson
C-----------------------------------------------------------------------
      FUNCTION HWH2T2(I,J,K,L,Z1,Z2,P1,P2)
C-----------------------------------------------------------------------
C     Returns the amplitude T2 from Nucl. Phys. B262 (1985) 235-262
C     I-L are the particles
C     Z1 and Z2 are the decay products of the Z
C     P1 is the polarization of the line I,J
C     P2 is the polarization of the gluon K
C-----------------------------------------------------------------------
      INCLUDE 'HERWIG65.INC'
      DOUBLE COMPLEX HWH2T2,S,D
      INTEGER I,J,K,L,Z1,Z2,P1,P2
      DOUBLE PRECISION B(6)
      DATA B/-1.0D0,-1.0D0,1.0D0,1.0D0,1.0D0,1.0D0/
      COMMON/HWHEWS/S(8,8,2),D(8,8)
      IF(P1.EQ.1.AND.P2.EQ.1) THEN
        HWH2T2 = FOUR*B(J)*S(I,Z2,1)*S(Z1,J,2)*S(J,K,1)*S(I,J,2)
      ELSEIF(P1.EQ.1.AND.P2.EQ.2) THEN
        HWH2T2 = FOUR*S(I,Z2,1)*S(K,J,2)*(B(J)*S(Z1,J,2)*S(J,I,1)
     &                                +B(K)*S(Z1,K,2)*S(K,I,1))
      ELSEIF(P1.EQ.2.AND.P2.EQ.1) THEN
        HWH2T2 = FOUR*S(I,Z1,2)*S(K,J,1)*(B(J)*S(Z2,J,1)*S(J,I,2)
     &                                +B(K)*S(Z2,K,1)*S(K,I,2))
      ELSEIF(P1.EQ.2.AND.P2.EQ.2) THEN
        HWH2T2 = FOUR*B(J)*S(I,Z1,2)*S(Z2,J,1)*S(J,K,2)*S(I,J,1)
      ELSE
        CALL HWWARN('HWH2T2',500,*999)
      ENDIF
 999  END
CDECK  ID>, HWH2T3.
*CMZ :-        -27/02/01  17:04:16  by  Peter Richardson
*-- Author :    Peter Richardson
C-----------------------------------------------------------------------
      FUNCTION HWH2T3(I,J,K,L,Z1,Z2,P1,P2)
C-----------------------------------------------------------------------
C     Returns the amplitude T3 from Nucl. Phys. B262 (1985) 235-262
C     I-L are the particles
C     Z1 and Z2 are the decay products of the Z
C     P1 is the polarization of the line I,J
C     P2 is the polarization of the gluon K
C-----------------------------------------------------------------------
      INCLUDE 'HERWIG65.INC'
      DOUBLE COMPLEX HWH2T3,S,D
      INTEGER I,J,K,L,Z1,Z2,P1,P2
      DOUBLE PRECISION B(6)
      DATA B/-1.0D0,-1.0D0,1.0D0,1.0D0,1.0D0,1.0D0/
      COMMON/HWHEWS/S(8,8,2),D(8,8)
      IF(P1.EQ.1.AND.P2.EQ.1) THEN
        HWH2T3 = FOUR*B(K)*S(I,K,1)*S(I,K,2)*S(K,Z2,1)*S(Z1,J,2)
      ELSEIF(P1.EQ.1.AND.P2.EQ.2) THEN
        HWH2T3 = ZERO
      ELSEIF(P1.EQ.2.AND.P2.EQ.1) THEN
        HWH2T3 = ZERO
      ELSEIF(P1.EQ.2.AND.P2.EQ.2) THEN
        HWH2T3 = FOUR*B(K)*S(I,K,2)*S(I,K,1)*S(K,Z1,2)*S(Z2,J,1)
      ELSE
        CALL HWWARN('HWH2T3',500,*999)
      ENDIF
 999  END
CDECK  ID>, HWH2T4
*CMZ :-        -27/02/01  17:04:16  by  Peter Richardson
*-- Author :    Peter Richardson
C-----------------------------------------------------------------------
      FUNCTION HWH2T4(I,J,K,L,Z1,Z2,P1,P2)
C-----------------------------------------------------------------------
C     Returns the amplitude T4 from Nucl. Phys. B262 (1985) 235-262
C     I-L are the particles
C     Z1 and Z2 are the decay products of the Z
C     P1 is the polarization of the line I,J
C     P2 is the polarization of the line K,L
C-----------------------------------------------------------------------
      INCLUDE 'HERWIG65.INC'
      DOUBLE COMPLEX HWH2T4,AP,AM,S,D
      INTEGER I,J,K,L,Z1,Z2,J1,J2,J3,J4,P1,P2
      DOUBLE PRECISION B(6)
      DATA B/-1.0D0,-1.0D0,1.0D0,1.0D0,1.0D0,1.0D0/
      COMMON/HWHEWS/S(8,8,2),D(8,8)
      AP(J1,J2,J3,J4) = FOUR*S(J1,Z2,1)*S(J4,J2,2)*
     &     (S(Z1,Z2,2)*S(Z2,J3,1)+B(J1)*S(Z1,J1,2)*S(J1,J3,1))
      AM(J1,J2,J3,J4) = FOUR*S(J1,Z1,2)*S(J4,J2,1)*
     &     (S(Z2,Z1,1)*S(Z1,J3,2)+B(J1)*S(Z2,J1,1)*S(J1,J3,2))
      IF(P1.EQ.1.AND.P2.EQ.1) THEN
        HWH2T4 = AP(I,J,K,L)
      ELSEIF(P1.EQ.1.AND.P2.EQ.2) THEN
        HWH2T4 = AP(I,J,L,K)
      ELSEIF(P1.EQ.2.AND.P2.EQ.1) THEN
        HWH2T4 = AM(I,J,L,K)
      ELSEIF(P1.EQ.2.AND.P2.EQ.2) THEN
        HWH2T4 = AM(I,J,K,L)
      ELSE
        CALL HWWARN('HWH2T4',500,*999)
      ENDIF
 999  END
CDECK  ID>, HWH2T5
*CMZ :-        -27/02/01  17:04:16  by  Peter Richardson
*-- Author :    Peter Richardson
C-----------------------------------------------------------------------
      FUNCTION HWH2T5(I,J,K,L,Z1,Z2,P1,P2)
C-----------------------------------------------------------------------
C     Returns the amplitude T5 from Nucl. Phys. B262 (1985) 235-262
C     I-L are the particles
C     Z1 and Z2 are the decay products of the Z
C     P1 is the polarization of the line I,J
C     P2 is the polarization of the line K,L
C-----------------------------------------------------------------------
      INCLUDE 'HERWIG65.INC'
      DOUBLE COMPLEX HWH2T5,AP,AM,S,D
      INTEGER I,J,K,L,Z1,Z2,J1,J2,J3,J4,P1,P2
      DOUBLE PRECISION B(6)
      DATA B/-1.0D0,-1.0D0,1.0D0,1.0D0,1.0D0,1.0D0/
      COMMON/HWHEWS/S(8,8,2),D(8,8)
      AP(J1,J2,J3,J4) = FOUR*S(J1,Z2,1)*S(J4,J2,2)*
     &     (S(Z1,Z2,2)*S(Z2,J3,1)+B(J1)*S(Z1,J1,2)*S(J1,J3,1))
      AM(J1,J2,J3,J4) = FOUR*S(J1,Z1,2)*S(J4,J2,1)*
     &     (S(Z2,Z1,1)*S(Z1,J3,2)+B(J1)*S(Z2,J1,1)*S(J1,J3,2))
      IF(P1.EQ.1.AND.P2.EQ.1) THEN
        HWH2T5 = AM(J,I,L,K)
      ELSEIF(P1.EQ.1.AND.P2.EQ.2) THEN
        HWH2T5 = AM(J,I,K,L)
      ELSEIF(P1.EQ.2.AND.P2.EQ.1) THEN
        HWH2T5 = AP(J,I,K,L)
      ELSEIF(P1.EQ.2.AND.P2.EQ.2) THEN
        HWH2T5 = AP(J,I,L,K)
      ELSE
        CALL HWWARN('HWH2T5',500,*999)
      ENDIF
 999  END
CDECK  ID>, HWH2T6
*CMZ :-        -27/02/01  17:04:16  by  Peter Richardson
*-- Author :    Peter Richardson
C-----------------------------------------------------------------------
      FUNCTION HWH2T6(I,J,K,L,Z1,Z2,P1,P2,P3)
C-----------------------------------------------------------------------
C     Returns the amplitude T6 from Nucl. Phys. B262 (1985) 235-262
C     I-L are the particles
C     Z1 and Z2 are the decay products of the Z
C     P1 is the polarization of the line I,J
C     P2 is the polarization of the gluon K
C     P3 is the polarization of the gluon L
C-----------------------------------------------------------------------
      INCLUDE 'HERWIG65.INC'
      DOUBLE COMPLEX HWH2T6,S,D
      INTEGER I,J,K,L,Z1,Z2,J1,J2,P1,P2,P3
      DOUBLE PRECISION B(6)
      DATA B/-1.0D0,-1.0D0,1.0D0,1.0D0,1.0D0,1.0D0/
      COMMON/HWHEWS/S(8,8,2),D(8,8)
      IF(P1.EQ.1) THEN
         J1 = Z1
         J2 = Z2
      ELSE
         J1 = Z2
         J2 = Z1
      ENDIF
      IF((P1.EQ.1.AND.P2.EQ.1.AND.P3.EQ.1).OR.
     &   (P1.EQ.2.AND.P2.EQ.2.AND.P3.EQ.2)) THEN
        HWH2T6 = 8.0D0*B(J)*S(I,J2,1)*D(L,J)*S(K,J,2)*
     &             (B(J)*S(J1,J,2)*S(J,K,1)+B(L)*S(J1,L,2)*S(L,K,1))
      ELSEIF((P1.EQ.1.AND.P2.EQ.1.AND.P3.EQ.2).OR.
     &       (P1.EQ.2.AND.P2.EQ.2.AND.P3.EQ.1)) THEN
        HWH2T6 = 8.0D0*B(J)*S(I,J2,1)*S(L,J,2)*S(J,K,1)*S(L,J,2)*
     &            (B(J)*S(J1,J,2)*S(J,K,1)+B(L)*S(J1,L,2)*S(L,K,1))
      ELSEIF((P1.EQ.1.AND.P2.EQ.2.AND.P3.EQ.1).OR.
     &       (P1.EQ.2.AND.P2.EQ.1.AND.P3.EQ.2)) THEN
        HWH2T6 = 8.0D0*B(J)*S(I,J2,1)*S(K,J,2)*S(J,L,1)*S(K,J,2)*
     &            (B(J)*S(J1,J,2)*S(J,L,1)+B(K)*S(J1,K,2)*S(K,L,1))
      ELSEIF((P1.EQ.1.AND.P2.EQ.2.AND.P3.EQ.2).OR.
     &       (P1.EQ.2.AND.P2.EQ.1.AND.P3.EQ.1)) THEN
        HWH2T6 = 8.0D0*S(I,J2,1)*S(L,J,2)*(B(J)*D(K,J)+B(L)*D(K,L))*
     &             (B(J)*S(J1,J,2)*S(J,L,1)+B(K)*S(J1,K,2)*S(K,L,1))
      ELSE
        CALL HWWARN('HWH2T6',500,*999)
      ENDIF
      IF(P1.EQ.2) HWH2T6 = DCONJG(HWH2T6)
 999  END
CDECK  ID>, HWH2T7
*CMZ :-        -27/02/01  17:04:16  by  Peter Richardson
*-- Author :    Peter Richardson
C-----------------------------------------------------------------------
      FUNCTION HWH2T7(I,J,K,L,Z1,Z2,P1,P2,P3)
C-----------------------------------------------------------------------
C     Returns the amplitude T7 from Nucl. Phys. B262 (1985) 235-262
C     I-L are the particles
C     Z1 and Z2 are the decay products of the Z
C     P1 is the polarization of the line I,J
C     P2 is the polarization of the gluon K
C     P3 is the polarization of the gluon L
C-----------------------------------------------------------------------
      INCLUDE 'HERWIG65.INC'
      DOUBLE COMPLEX HWH2T7,S,D
      INTEGER I,J,K,L,Z1,Z2,J1,J2,P1,P2,P3
      DOUBLE PRECISION B(6)
      DATA B/-1.0D0,-1.0D0,1.0D0,1.0D0,1.0D0,1.0D0/
      COMMON/HWHEWS/S(8,8,2),D(8,8)
      IF(P1.EQ.1) THEN
        J1 = Z1
        J2 = Z2
      ELSE
        J1 = Z2
        J2 = Z1
      ENDIF
      IF((P1.EQ.1.AND.P2.EQ.1.AND.P3.EQ.1).OR.
     &   (P1.EQ.2.AND.P2.EQ.2.AND.P3.EQ.2)) THEN
        HWH2T7 = 8.0D0*B(J)*S(I,K,1)*S(J1,J,2)*S(J,L,1)*S(K,J,2)*
     &                 (B(I)*S(L,I,2)*S(I,J2,1)+B(K)*S(L,K,2)*S(K,J2,1))
      ELSEIF((P1.EQ.1.AND.P2.EQ.1.AND.P3.EQ.2).OR.
     &       (P1.EQ.2.AND.P2.EQ.2.AND.P3.EQ.1)) THEN
        HWH2T7 = 8.0D0*S(I,K,1)*S(L,J,2)*
     &                (B(I)*S(L,I,2)*S(I,J2,1)+B(K)*S(L,K,2)*S(K,J2,1))*
     &                (B(J)*S(J1,J,2)*S(J,K,1)+B(L)*S(J1,L,2)*S(L,K,1))
      ELSEIF((P1.EQ.1.AND.P2.EQ.2.AND.P3.EQ.1).OR.
     &       (P1.EQ.2.AND.P2.EQ.1.AND.P3.EQ.2)) THEN
        HWH2T7 = 8.0D0*B(I)*B(J)*S(I,L,1)*S(K,I,2)*
     &        S(I,J2,1)*S(J1,J,2)*S(J,L,1)*S(K,J,2)
      ELSEIF((P1.EQ.1.AND.P2.EQ.2.AND.P3.EQ.2).OR.
     &       (P1.EQ.2.AND.P2.EQ.1.AND.P3.EQ.1)) THEN
        HWH2T7 = 8.0D0*B(I)*S(I,L,1)*S(K,I,2)*S(I,J2,1)*S(L,J,2)*
     &                 (B(J)*S(J1,J,2)*S(J,K,1)+B(L)*S(J1,L,2)*S(L,K,1))
      ELSE
        CALL HWWARN('HWH2T7',500,*999)
      ENDIF
      IF(P1.EQ.2) HWH2T7 = DCONJG(HWH2T7)
 999  END
CDECK  ID>, HWH2T8
*CMZ :-        -27/02/01  17:04:16  by  Peter Richardson
*-- Author :    Peter Richardson
C-----------------------------------------------------------------------
      FUNCTION HWH2T8(I,J,K,L,Z1,Z2,P1,P2,P3)
C-----------------------------------------------------------------------
C     Returns the amplitude T8 from Nucl. Phys. B262 (1985) 235-262
C     I-L are the particles
C     Z1 and Z2 are the decay products of the Z
C     P1 is the polarization of the line I,J
C     P2 is the polarization of the gluon K
C     P3 is the polarization of the gluon L
C-----------------------------------------------------------------------
      INCLUDE 'HERWIG65.INC'
      DOUBLE COMPLEX HWH2T8,S,D
      INTEGER I,J,K,L,Z1,Z2,J1,J2,P1,P2,P3
      DOUBLE PRECISION B(6)
      DATA B/-1.0D0,-1.0D0,1.0D0,1.0D0,1.0D0,1.0D0/
      COMMON/HWHEWS/S(8,8,2),D(8,8)
      IF(P1.EQ.1) THEN
        J1 = Z1
        J2 = Z2
      ELSE
        J1 = Z2
        J2 = Z1
      ENDIF
      IF((P1.EQ.1.AND.P2.EQ.1.AND.P3.EQ.1).OR.
     &   (P1.EQ.2.AND.P2.EQ.2.AND.P3.EQ.2)) THEN
        HWH2T8 = 8.0D0*S(I,K,1)*S(J1,J,2)*(B(I)*D(L,I)+B(K)*D(L,K))*
     &                (B(I)*S(K,I,2)*S(I,J2,1)+B(L)*S(K,L,2)*S(L,J2,1))
      ELSEIF((P1.EQ.1.AND.P2.EQ.1.AND.P3.EQ.2).OR.
     &       (P1.EQ.2.AND.P2.EQ.2.AND.P3.EQ.1)) THEN
        HWH2T8 = 8.0D0*B(I)*S(I,K,1)*S(L,I,2)*S(I,K,1)*S(J1,J,2)*
     &                 (B(I)*S(L,I,2)*S(I,J2,1)+B(K)*S(L,K,2)*S(K,J2,1))
      ELSEIF((P1.EQ.1.AND.P2.EQ.2.AND.P3.EQ.1).OR.
     &       (P1.EQ.2.AND.P2.EQ.1.AND.P3.EQ.2)) THEN
        HWH2T8 = 8.0D0*B(I)*S(I,L,1)*S(K,I,2)*S(I,L,1)*S(J1,J,2)*
     &                 (B(I)*S(K,I,2)*S(I,J2,1)+B(L)*S(K,L,2)*S(L,J2,1))
      ELSEIF((P1.EQ.1.AND.P2.EQ.2.AND.P3.EQ.2).OR.
     &       (P1.EQ.2.AND.P2.EQ.1.AND.P3.EQ.1)) THEN
        HWH2T8 = 8.0D0*B(I)*S(I,L,1)*D(I,K)*S(J1,J,2)*
     &                 (B(I)*S(L,I,2)*S(I,J2,1)+B(K)*S(L,K,2)*S(K,J2,1))
      ELSE
        CALL HWWARN('HWH2T8',500,*999)
      ENDIF
      IF(P1.EQ.2) HWH2T8 = DCONJG(HWH2T8)
 999  END
CDECK  ID>, HWH2T9
*CMZ :-        -27/02/01  17:04:16  by  Peter Richardson
*-- Author :    Peter Richardson
C-----------------------------------------------------------------------
      FUNCTION HWH2T9(I,J,K,L,Z1,Z2,P1,P2,P3)
C-----------------------------------------------------------------------
C     Returns the amplitude T9 from Nucl. Phys. B262 (1985) 235-262
C     N.B. DELTA FUNCTION FOR THE GLUON POLARIZATIONS HERE
C     I-L are the particles
C     Z1 and Z2 are the decay products of the Z
C     P1 is the polarization of the line I,J
C     P2 is the polarization of the gluon K
C     P3 is the polarization of the gluon L
C-----------------------------------------------------------------------
      INCLUDE 'HERWIG65.INC'
      DOUBLE COMPLEX HWH2T9,S,D
      INTEGER I,J,K,L,Z1,Z2,J1,J2,P1,P2,P3
      DOUBLE PRECISION B(6)
      DATA B/-1.0D0,-1.0D0,1.0D0,1.0D0,1.0D0,1.0D0/
      COMMON/HWHEWS/S(8,8,2),D(8,8)
      IF(P2.NE.P3) THEN
         HWH2T9 = ZERO
      ELSE
        IF(P1.EQ.1) THEN
          J1 = Z1
          J2 = Z2
        ELSEIF(P1.EQ.2) THEN
          J1 = Z2
          J2 = Z1
        ENDIF
        HWH2T9 = TWO*S(I,J2,1)*(
     &           B(K)*S(K,J,2)*(B(J)*S(J1,J,2)*S(J,K,1)
     &                           +B(L)*S(J1,L,2)*S(L,K,1))
     &          -B(L)*S(L,J,2)*(B(J)*S(J1,J,2)*S(J,L,1)
     &                           +B(K)*S(J1,K,2)*S(K,L,1)))
        IF(P1.EQ.2) HWH2T9 = DCONJG(HWH2T9)
      ENDIF
 999  END
CDECK  ID>, HWH2T0
*CMZ :-        -27/02/01  17:04:16  by  Peter Richardson
*-- Author :    Peter Richardson
C-----------------------------------------------------------------------
      FUNCTION HWH2T0(I,J,K,L,Z1,Z2,P1,P2,P3)
C-----------------------------------------------------------------------
C     Returns the amplitude T10 from Nucl. Phys. B262 (1985) 235-262
C     N.B. DELTA FUNCTION FOR THE GLUON POLARIZATIONS HERE
C     I-L are the particles
C     Z1 and Z2 are the decay products of the Z
C     P1 is the polarization of the line I,J
C     P2 is the polarization of the gluon K
C     P3 is the polarization of the gluon L
C-----------------------------------------------------------------------
      INCLUDE 'HERWIG65.INC'
      DOUBLE COMPLEX HWH2T0,S,D
      INTEGER I,J,K,L,Z1,Z2,J1,J2,P1,P2,P3
      DOUBLE PRECISION B(6)
      DATA B/-1.0D0,-1.0D0,1.0D0,1.0D0,1.0D0,1.0D0/
      COMMON/HWHEWS/S(8,8,2),D(8,8)
      IF(P2.NE.P3) THEN
         HWH2T0 = ZERO
      ELSE
        IF(P1.EQ.1) THEN
          J1 = Z1
          J2 = Z2
        ELSEIF(P1.EQ.2) THEN
          J1 = Z2
          J2 = Z1
        ENDIF
        HWH2T0 = TWO*S(J1,J,2)*(
     &            B(K)*S(I,K,1)*(B(I)*S(K,I,2)*S(I,J2,1)
     &                           +B(L)*S(K,L,2)*S(L,J2,1))
     &           -B(L)*S(I,L,1)*(B(I)*S(L,I,2)*S(I,J2,1)
     &                           +B(K)*S(L,K,2)*S(K,J2,1)))
        IF(P1.EQ.2) HWH2T0 = DCONJG(HWH2T0)
      ENDIF
 999  END
CDECK  ID>, HWH2VH.
*CMZ :-        -26/11/00  17.21.55  by  Bryan Webber
*-- Author :  Stefano Moretti
C-----------------------------------------------------------------------
      SUBROUTINE HWH2VH(P1,P2,P3,P4,RMV,RES,RESL,REST)
C-----------------------------------------------------------------------
C...Matrix element for q(1) + q(')-bar(2) -> V(3) + Higgs(4),
C...V=Z(W+/-), all masses retained (but no Yukawa couplings to quarks).
C...It factorises 64.*PIFAC**3*ALPHA**3/4./SWEIN/SWEIN/SWEIN/EMW/EMW
C...times:
C...         (VQ*VQ+AQ*AQ)/(1.-SWEIN)/(1.-SWEIN)    if V=Z
C...         VCKM(q,q')                             if V=W+/-
C
C...First release:  1-APR-1998 by Stefano Moretti
C-----------------------------------------------------------------------
      IMPLICIT NONE
      DOUBLE PRECISION P1(0:3),P2(0:3),P3(0:3),P4(0:3)
      DOUBLE PRECISION P(0:3)
      DOUBLE PRECISION RMV,GAMV,RES,RESL,REST
      INTEGER I
      DOUBLE PRECISION S,S12,S13,S23
      DOUBLE PRECISION T,    T13,T23
      DOUBLE PRECISION PV,CFC
      PARAMETER (GAMV=0.D0)
      S=(P1(0)+P2(0))**2
      DO I=1,3
        S=S-(P1(I)+P2(I))**2
      END DO
      S12=P1(0)*P2(0)
      S13=P1(0)*P3(0)
      S23=P2(0)*P3(0)
      DO I=1,3
        S12=S12-P1(I)*P2(I)
        S13=S13-P1(I)*P3(I)
        S23=S23-P2(I)*P3(I)
      END DO
C...Total ME.
      RES=(S12+2.D0/RMV/RMV*(S13*S23))
     &   /((S-RMV**2)**2+GAMV**2*RMV**2)
     &   /12.D0
C...Extracts spin dependence.
      PV=SQRT(P3(1)**2+P3(2)**2+P3(3)**2)
      CFC=P3(0)/PV
      DO I=1,3
        P(I)=P3(I)*CFC
      END DO
      P(0)=PV**2/P3(0)*CFC
      T=P(0)**2
      DO I=1,3
        T=T-P(I)**2
      END DO
      T13=P1(0)*P(0)
      T23=P2(0)*P(0)
      DO I=1,3
        T13=T13-P1(I)*P(I)
        T23=T23-P2(I)*P(I)
      END DO
C...Longitudinal ME (along V direction).
      RESL=(2.D0/RMV/RMV*(T13*T23)-S12*T/RMV/RMV)
     &    /((S-RMV**2)**2+GAMV**2*RMV**2)
     &    /12.D0
C...Transverse ME (perpendicular to V direction).
      REST=RES-RESL
      RETURN
      END
