C
C
      SUBROUTINE CHAINS(IMODE,IREJ)
C********************************************************************
C
C    calculation of chain combinatorics, Lorentz boosts and
C                   particle codes
C
C                - splitting of gluons
C                - chains will be built up from pairs of partons
C                  according to their color labels
C                  with IDHEP(..) = -1
C                - there can be other particles between to chain partons
C                  (these will be unchanged by chain construction)
C
C    input:      IMODE    1  complete chain processing
C                        -1 initialization
C                        -2 output of statistics
C
C    output:     COMMON  STRING
C                     50   rejection due to user cutoffs
C                IREJ    1 combination of chains impossible
C                        0 successful combination
C                       50 rejection due to user cutoffs
C
C********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      PARAMETER (ZERO   =  0.D0,
     &           ONE    =  1.D0,
     &           TWO    =  2.D0,
     &           OHALF  =  0.5D0,
     &           DEPS   =  1.D-40,
     &           EPS    =  1.D-5)
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C
      PARAMETER (NMXJ=40)
      CHARACTER*10 REJTIT
      COMMON /REJSTA/ IFAIL(NMXJ),REJTIT(NMXJ)
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
      INTEGER MXSECT
      PARAMETER ( MAXPRO = 16, MAXTAB = 20 )
      COMMON /HAXSEC/ XSECTA(4,-1:MAXPRO,4,MAXTAB),XSECT(6,-1:MAXPRO),
     &                MXSECT(0:4,-1:MAXPRO,4),ECMSH(4,MAXTAB),ISTTAB
C
      PARAMETER (NMXHEP=2000)
      COMMON/HEPEVS/NEVHEP,NHEP,ISTHEP(NMXHEP),IDHEP(NMXHEP),
     &JMOHEP(2,NMXHEP),JDAHEP(2,NMXHEP),PHEP(5,NMXHEP),VHEP(4,NMXHEP)
      COMMON/HEPEVE/IMPART(NMXHEP),IPHIST(2,NMXHEP),ICOLOR(2,NMXHEP)
      PARAMETER (MSTR=100)
      COMMON /STRING/ GAMBET(4,MSTR),NPOS(3,MSTR),NCODE(MSTR),
     &     IPAR1(MSTR),IPAR2(MSTR),IPAR3(MSTR),IPAR4(MSTR),
     &     NNCH(MSTR),IBHAD(MSTR),ISTR
C
      COMMON/POMRES/IPOPOS(2,100),IPORES(100),IPOIX1,IPOIX2
C
*     CHARACTER*72 TITLE
*     DIMENSION XLIMB(2)
C
      IREJ = 0
      IF(IMODE.EQ.-1) THEN
        CALL POMCOR(-1)
        CALL MASCOR(-1)
        CALL PARCOR(-1,IREJ)
C  pt distribution of partons soft
*       CALL NEWHIS(ZERO,5.D0,ZERO,XLIMB,30,IPTSA)
C  pt distribution of partons hard (inside chains)
*       CALL NEWHIS(ZERO,10.D0,ZERO,XLIMB,30,IPTHI)
C  pt distribution of partons hard (chain ends)
*       CALL NEWHIS(ZERO,5.D0,ZERO,XLIMB,30,IPTHO)
C  pt distribution of all partons
*       CALL NEWHIS(ZERO,10.D0,ZERO,XLIMB,40,IPTA)
        RETURN
      ELSE IF(IMODE.EQ.-2) THEN
        CALL POMCOR(-2)
        CALL MASCOR(-2)
        CALL PARCOR(-2,IREJ)
C  pt distribution of partons
*       TITLE = 'PRIMARY PARTON PT DISTRIBUTION (SOFT CHAINS)'
*       CALL OUTHIS(IPTSA,1,TITLE,ONE,1)
C  pt distribution of partons
*       TITLE = 'PRIMARY PARTON PT DISTRIBUTION (HARD CHAINS INSIDE)'
*       CALL OUTHIS(IPTHI,1,TITLE,ONE,1)
C  pt distribution of partons
*       TITLE = 'PRIMARY PARTON PT DISTRIBUTION (HARD CHAIN ENDS)'
*       CALL OUTHIS(IPTHO,1,TITLE,ONE,1)
C  pt distribution of all partons
*       TITLE = 'PRIMARY PARTON PT DISTRIBUTION (ALL)'
*       CALL OUTHIS(IPTA,1,TITLE,ONE,1)
        RETURN
      ENDIF
C
C  generate enhanced graphs
      IF(IPOIX2.GT.0) THEN
 200    CONTINUE
        I1 = IPOIX1
        I2 = IPOIX2
        IF(ISWMDL(14).EQ.1) IPOIX1 = 0
        KSPOMS = KSPOM
        KSREGS = KSREG
        KHPOMS = KHPOM
        KHDIRS = KHDIR
        DO 110 I=I1,I2
          KSPOM = 0
          KSREG = 0
          KHPOM = 0
          KHDIR = 0
          IF(IPORES(I).EQ.8) THEN
            KSPOM = 2
            LSPOM = 2
            LHPOM = 0
            LSREG = 0
            LHDIR = 0
            CALL STDPAR(IPOPOS(1,I),IPOPOS(2,I),LSPOM,LSREG,LHPOM,
     &        LHDIR,IREJ)
            IF(IREJ.NE.0) THEN
              IF(IDEB(4).GE.2) THEN
                WRITE(6,'(/1X,A,I5)')
     &            'CHAINS:WARNING:SEC.REJECTION BY STDPAR',IREJ
                CALL POPREV(-1)
              ENDIF
              RETURN
            ENDIF
            KSPOM = KSPOMS+LSPOM
            KSREG = KSREGS+LSREG
            KHPOM = KHPOMS+LHPOM
            KHDIR = KHDIRS+LHDIR
          ELSE IF(IPORES(I).EQ.4) THEN
            ITEMP = ISWMDL(25)
            ISWMDL(25) = 0
            CALL PODBLE(IPOPOS(1,I),IPOPOS(2,I),MSOFT,MHARD,1,IREJ)
            ISWMDL(25) = ITEMP
            IF(IREJ.NE.0) THEN
              IF(IDEB(4).GE.2) THEN
                WRITE(6,'(/1X,A,I5)')
     &            'CHAINS:WARNING:SEC.REJECTION BY PODBLE',IREJ
                CALL POPREV(-1)
              ENDIF
              RETURN
            ENDIF
            KSDPO = KSDPO+1
            KSPOM = KSPOMS+KSPOM
            KSREG = KSREGS+KSREG
            KHPOM = KHPOMS+KHPOM
            KHDIR = KHDIRS+KHDIR
          ELSE
            IDIF1 = 1
            IDIF2 = 1
            IF(IPORES(I).EQ.5) THEN
              IDIF2 = 0
              KSTRG = KSTRG+1
            ELSE IF(IPORES(I).EQ.6) THEN
              IDIF1 = 0
              KSTRG = KSTRG+1
            ELSE
              KSLOO = KSLOO+1
            ENDIF
            ITEMP = ISWMDL(16)
            ISWMDL(16) = 0
            CALL PODIFF(IDIF1,IDIF2,IPOPOS(1,I),IPOPOS(2,I),
     &        MSOFT,MHARD,0,IREJ)
            ISWMDL(16) = ITEMP
            IF(IREJ.NE.0) THEN
              IF(IDEB(4).GE.2) THEN
                WRITE(6,'(/1X,A,I5)')
     &            'CHAINS:WARNING:SEC.REJECTION BY PODIFF',IREJ
                CALL POPREV(-1)
              ENDIF
              RETURN
            ENDIF
            KSPOM = KSPOMS+KSPOM
            KSREG = KSREGS+KSREG
            KHPOM = KHPOMS+KHPOM
            KHDIR = KHDIRS+KHDIR
          ENDIF
 110    CONTINUE
        IF(IPOIX2.GT.I2) THEN
          IPOIX1 = I2+1
          GOTO 200
        ENDIF
      ENDIF
C
C  split gluons to q qbar pairs
      IF(ISWMDL(9).GT.0) THEN
        NHEPO = NHEP
        DO 30 I=3,NHEPO
          IF((ISTHEP(I).EQ.-1).AND.(IDHEP(I).EQ.21)) THEN
            ICG1=ICOLOR(1,I)
            ICG2=ICOLOR(2,I)
            IQ1 = 0
            IQ2 = 0
            DO 40 K=3,NHEPO
              IF(ICOLOR(1,K).EQ.-ICG1) THEN
                IQ1 = K
                IF(IQ1*IQ2.NE.0) GOTO 45
              ELSE IF(ICOLOR(1,K).EQ.-ICG2) THEN
                IQ2 = K
                IF(IQ1*IQ2.NE.0) GOTO 45
              ENDIF
 40         CONTINUE
            WRITE(6,'(/1X,A,3I6)')
     &        'CHAINS:ERROR:NO MATCHING COLOR FOUND (IG,ICG1,ICG2)',
     &        I,ICG1,ICG2
            CALL POABRT
 45         CONTINUE
            CALL GLU2QU(I,IQ1,IQ2,IREJ)
            IF(IREJ.NE.0) THEN
              IF(IDEB(19).GE.5) THEN
                WRITE(6,'(/,1X,A)')
     &            'CHAINS:WARNING:NO GLUON SPLITTING POSSIBLE'
                CALL POPREV(0)
              ENDIF
              RETURN
            ENDIF
          ENDIF
 30     CONTINUE
      ENDIF
C
C  construct chains and write entries sorted by chains
      ISTR = ISTR+1
      NHEPO = NHEP
      DO 50 I=3,NHEPO
        IF(ISTHEP(I).EQ.1) THEN
C  hadrons
          NPOS(1,ISTR) = I
          NPOS(2,ISTR) = 0
          NPOS(3,ISTR) = 0
          NCODE(ISTR) = -99
          IPHIST(1,I) = ISTR
          ISTR = ISTR+1
        ELSE IF((ISTHEP(I).EQ.-1).AND.(IDHEP(I).NE.21)) THEN
C  partons
          ICOL1 = -ICOLOR(1,I)
          P1 = PHEP(1,I)
          P2 = PHEP(2,I)
          P3 = PHEP(3,I)
          P4 = PHEP(4,I)
          ICH1 = IPOCH3(I,2)
          IBA1 = IPOBA3(I,2)
          CALL REGPAR(-1,IDHEP(I),IMPART(I),I,0,
     &                P1,P2,P3,P4,IPHIST(1,I),IPHIST(2,I),
     &                ICOLOR(1,I),ICOLOR(2,I),IPOS,1)
          JM1 = IPOS
C
 65       CONTINUE
          NRPOM = 0
          DO 55 K=1,NHEPO
            IF(ISTHEP(K).EQ.-1)THEN
              IF(IDHEP(K).EQ.21) THEN
                IF(ICOLOR(1,K).EQ.ICOL1) THEN
                  ICOL1 = -ICOLOR(2,K)
                  GOTO 60
                ELSE IF(ICOLOR(2,K).EQ.ICOL1) THEN
                  ICOL1 = -ICOLOR(1,K)
                  GOTO 60
                ENDIF
              ELSE IF(ICOLOR(1,K).EQ.ICOL1) THEN
                ICOL1 = 0
                GOTO 60
              ENDIF
            ENDIF
 55       CONTINUE
          WRITE(6,'(/1X,A,I5)')
     &      'CHAINS:ERROR:NO MATCHING PART. FOUND FOR COLOR',-ICOL1
          CALL POABRT
 60       CONTINUE
          P1 = P1+PHEP(1,K)
          P2 = P2+PHEP(2,K)
          P3 = P3+PHEP(3,K)
          P4 = P4+PHEP(4,K)
          NRPOM = MAX(NRPOM,IPHIST(1,K))
          ICH1 = ICH1+IPOCH3(K,2)
          IBA1 = IBA1+IPOBA3(K,2)
          CALL REGPAR(-1,IDHEP(K),IMPART(K),K,0,
     &      PHEP(1,K),PHEP(2,K),PHEP(3,K),PHEP(4,K),
     &      IPHIST(1,K),IPHIST(2,K),ICOLOR(1,K),ICOLOR(2,K),IPOS,1)
C  further parton involved?
          IF(ICOL1.NE.0) GOTO 65
          JM2 = IPOS
C  register chain
          IF(NRPOM.GT.100) NRPOM = NRPOM-MOD(NRPOM,100)
          CALL REGPAR(-1,8888,0,JM1,-JM2,P1,P2,P3,P4,
     &                ISTR,NRPOM,ICH1,IBA1,IPOS,1)
C  store additional chain information
          NPOS(1,ISTR) = IPOS
          NPOS(2,ISTR) = JM1
          NPOS(3,ISTR) = -JM2
C  calculate BAMJET chain codes
          CALL ID2BAM(IDHEP(JM1),IDHEP(JM2),NCODE(ISTR),
     &                IPAR1(ISTR),IPAR2(ISTR),IPAR3(ISTR),IPAR4(ISTR))
          ISTR = ISTR+1
        ENDIF
 50   CONTINUE
      ISTR = ISTR-1
C
C  statistics of partons before mass correction
*     DO 53 K=NHEPO+1,NHEP
*        IF(IDHEP(K).EQ.8888) GOTO 54
*        PT = SQRT(PHEP(1,K)**2+PHEP(2,K)**2)
C  pt distribution of soft partons
*        IF(ABS(IPHIST(1,K)).LT.100) CALL FILHIS(PT,ONE,IPTSA)
C  pt distribution of hard partons inside of chains
*        IF(IPHIST(1,K).LE.-100) CALL FILHIS(PT,ONE,IPTHI)
C  pt distribution of hard partons at the chain ends
*        IF(IPHIST(1,K).GE.100) CALL FILHIS(PT,ONE,IPTHO)
C  pt distribution of all partons
*        CALL FILHIS(PT,ONE,IPTA)
*54      CONTINUE
*53   CONTINUE
*     CALL ADDHIS(IPTSA)
*     CALL ADDHIS(IPTHI)
*     CALL ADDHIS(IPTHO)
*     CALL ADDHIS(IPTA)
C
      IF(IDEB(19).GE.17) THEN
        WRITE(6,'(1X,A)') 'CHAINS:DEBUG:AFTER CHAIN CONSTRUCTION'
        CALL POPREV(0)
      ENDIF
C
C  Pomeron corrections
      CALL POMCOR(IREJ)
      IF(IREJ.NE.0) THEN
        IFAIL(27) = IFAIL(27)+1
        IF(IDEB(19).GE.3) THEN
          WRITE(6,'(1X,A,I6)')
     &      'CHAINS:WARNING:POMCOR REJECTION (IREJ)',IREJ
          CALL POPREV(-1)
        ENDIF
        RETURN
      ENDIF
C
C  chain mass corrections
      CALL MASCOR(IREJ)
      IF(IREJ.NE.0) THEN
        IFAIL(34) = IFAIL(34)+1
        IF(IDEB(19).GE.3) THEN
          WRITE(6,'(1X,A,I6)')
     &      'CHAINS:WARNING:MASCOR REJECTION (IREJ)',IREJ
          CALL POPREV(-1)
        ENDIF
        RETURN
      ENDIF
C
C  parton mass corrections
      DO 100 I=1,ISTR
        IF(NCODE(I).GE.0) THEN
          CALL PARCOR(NPOS(1,I),IREJ)
          IF(IREJ.NE.0) THEN
            IFAIL(35) = IFAIL(35)+1
            IF(IDEB(19).GE.3) THEN
              WRITE(6,'(1X,A,I6)')
     &          'CHAINS:WARNING:PARCOR REJECTION (IREJ)',IREJ
              CALL POPREV(-1)
            ENDIF
            RETURN
          ENDIF
        ENDIF
 100  CONTINUE
C
C  statistics of hard processes
      DO 550 I=3,NHEP
        IF(ISTHEP(I).EQ.25) THEN
          K  = IMPART(I)
          II = IDHEP(I)
          MXSECT(4,K,II) = MXSECT(4,K,II)+1
        ENDIF
 550  CONTINUE
C  debug: write out chains
      IF(IDEB(19).GE.5) THEN
        IF(IDEB(19).GE.10)
     &    CALL POQCHK(1,IDEV)
        IF(IDEB(19).GE.15) THEN
          CALL POPREV(0)
        ELSE
          CALL POPRST
        ENDIF
      ENDIF
      END
