C
C
      SUBROUTINE CMSKIN(IDX1,IDX2,CMVAL,IMODE)
C**********************************************************************
C
C     transformations to CMS system
C
C     input:
C            IMODE  1       correct Lorentz transformation
C                   2       Lorentz transformation neglecting particle
C                           mass
C                   3       Lorentz transformation assuming PI mass
C                           for all particles
C                  -1       initialization (IDX1 for particle 1 in pos.
C                           z direction, including rotations)
C            IDX1,IDX2      particle index in HEPEVS
C
C     output:
C            CMVAL(1-4)     four momentum
C            CMVAL(5)       rapidity
C            CMVAL(6)       pseudorapidity
C            CMVAL(7)       pt
C            CMVAL(8)       et
C            CMVAL(9)       xf
C            CMVAL(10)      weight for xf plots
C
C********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      SAVE
C
      PARAMETER ( PI    = 3.14159265359D0,
     &            TWOPI = 6.28318530718D0,
     &            DEPS  = 1.D-14,
     &            ZERO  = 0.D0,
     &            OHALF = 0.5D0,
     &            ONE   = 1.D0,
     &            TWO   = 2.D0 )
      DIMENSION  CMVAL(10)
C
      PARAMETER (NMXHEP=2000)
      COMMON/HEPEVS/NEVHEP,NHEP,ISTHEP(NMXHEP),IDHEP(NMXHEP),
     &JMOHEP(2,NMXHEP),JDAHEP(2,NMXHEP),PHEP(5,NMXHEP),VHEP(4,NMXHEP)
      COMMON/HEPEVE/IMPART(NMXHEP),IPHIST(2,NMXHEP),ICOLOR(2,NMXHEP)
C  CM system
      COMMON /LOCCMS/ ECM,PCM,GAMBET(4),SID,COD,SIF,COF,NPOS(2)
C
      DIMENSION PC(4)
C
      IF(IMODE.GT.0) THEN
C  transformation into CMS
        IF(IMODE.EQ.1) THEN
          E=PHEP(4,IDX1)
        ELSE IF(IMODE.EQ.2) THEN
          E=SQRT(PHEP(1,IDX1)**2+PHEP(2,IDX1)**2+PHEP(3,IDX1)**2)
        ELSE IF(IMODE.EQ.3) THEN
          E=SQRT(PHEP(1,IDX1)**2+PHEP(2,IDX1)**2+PHEP(3,IDX1)**2+PIM2)
        ELSE
          WRITE(6,'(/1X,A,I4)') 'CMSKIN:ERROR: UNSUPPORTED MODE',IMODE
          E=PHEP(4,IDX1)
        ENDIF
        CALL ALTRA(GAMBET(4),GAMBET(1),GAMBET(2),GAMBET(3),
     &             PHEP(1,IDX1),PHEP(2,IDX1),PHEP(3,IDX1),
     &             E,PTOT1,PC(1),PC(2),PC(3),PC(4))
        CALL TRANI(PC(1),PC(2),PC(3),COD,SID,COF,SIF,X,Y,Z)
        E = PC(4)
C  new kinematics
        YY  = -99999.D0
        ETA = -99999.D0
        ET  = 0.D0
        PT2 = X**2+Y**2
        PABS = SQRT(PT2+Z**2)
        PT = SQRT(PT2)
        IF(PABS.LT.1.D-8) GOTO 99
        THETA = ACOS(Z/PABS)
        IF((THETA.LT.1.D-8).OR.(PI-THETA.LT.1.D-8)) GOTO 99
C  pseudo rapidity
        ETA = -LOG(TAN(THETA/TWO))
C  transverse energy
        ET = E*SIN(THETA)
*       ET = SQRT(PABS**2+0.15D0**2)*SIN(THETA)
C  rapidity
        PPLUS  = E+Z
        PMINUS = E-Z
        IF((PPLUS*PMINUS).GT.DEPS) YY = OHALF*LOG(PPLUS/PMINUS)
 99     CONTINUE
C  weight of Feynman distributions
        XF = Z/PCM
        FEYNF = E/ECM
C  save kinematic variables
        CMVAL(1) = X
        CMVAL(2) = Y
        CMVAL(3) = Z
        CMVAL(4) = E
        CMVAL(5) = YY
        CMVAL(6) = ETA
        CMVAL(7) = PT
        CMVAL(8) = ET
        CMVAL(9) = XF
        CMVAL(10)= FEYNF
C
      ELSE IF(IMODE.EQ.-1) THEN
        PIM2 = 0.135D0**2
C  get  CM system
        PC(1) = PHEP(1,IDX1)+PHEP(1,IDX2)
        PC(2) = PHEP(2,IDX1)+PHEP(2,IDX2)
        PC(3) = PHEP(3,IDX1)+PHEP(3,IDX2)
        PC(4) = PHEP(4,IDX1)+PHEP(4,IDX2)
        SS = PC(4)**2-PC(1)**2-PC(2)**2-PC(3)**2
        ECM = SQRT(SS)
        DO 10 I=1,3
          GAMBET(I) = -PC(I)/ECM
 10     CONTINUE
        GAMBET(4) = PC(4)/ECM
        CALL ALTRA(GAMBET(4),GAMBET(1),GAMBET(2),GAMBET(3),
     &             PHEP(1,IDX1),PHEP(2,IDX1),PHEP(3,IDX1),
     &             PHEP(4,IDX1),PTOT1,PC(1),PC(2),PC(3),PC(4))
C  rotation angle: particle 1 moves along +z
        COD= PC(3)/PTOT1
        SID= SQRT((ONE-COD)*(ONE+COD))
        COF=ONE
        SIF=ZERO
        IF(PTOT1*SID.GT.DEPS) THEN
          COF=PC(1)/(SID*PTOT1)
          SIF=PC(2)/(SID*PTOT1)
          ANORF=SQRT(COF*COF+SIF*SIF)
          COF=COF/ANORF
          SIF=SIF/ANORF
        ENDIF
C  get CM momentum
        XM12 = PHEP(5,IDX1)**2
        XM22 = PHEP(5,IDX2)**2
        PCM = XLAMB(SS,XM12,XM22)/(2.D0*ECM)
        NPOS(1) = IDX1
        NPOS(2) = IDX2
      ELSE
        WRITE(6,'(1X,A,I7)') 'CMSKIN:ERROR:INVALID MODE',IMODE
        CALL POABRT
      ENDIF
      END
