C
C
      SUBROUTINE SOFTPT(ISOFT,PTCUT,PTMAX,XV,IV,PTSOF)
C***********************************************************************
C
C    select pt of soft chain ends
C
C    input:    ISOFT          number of soft partons
C                    -1       initialization
C                    >=0      sampling of p_t
C                    -2       output of statistics
C              PTCUT          cutoff for soft chains
C              PTMAX          maximal allowed PT
C              XV             field of x values
C              IV             0    sea quark
C                             1    valence quark
C
C    output:   COMMON XSECPT  containing parameters AAS,BETAS
C              PTSOF          filed with soft pt values
C
C    note:     ISWMDL(3/4) = 0  dNs/dP_t = P_t ASS * exp(-BETA*P_t**2)
C              ISWMDL(3/4) = 1  dNs/dP_t = P_t ASS * exp(-BETA*P_t)
C              ISWMDL(3/4) = 2  photon wave function
C              ISWMDL(3/4) = 10 no soft P_t assignment
C
C********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      PARAMETER (ZERO   =  0.D0,
     &           ONE    =  1.D0,
     &           TWO    =  2.D0,
     &           OHALF  =  0.5D0,
     &           DEPS   =  1.D-40)
C
      DIMENSION PTSOF(2,100),XV(100),IV(100)
C
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
C  CM system of Pomeron/Reggeon exchange
      COMMON /POMCMS/ ECMP,PCMP,PMASSP(2),PVIRTP(2),GAMBEP(4),
     &                SIDP,CODP,SIFP,COFP,NPOSP(2),
     &                IDPDG1,IDBAM1,IDPDG2,IDBAM2
C
      COMMON /HASCA / PTWANT,AS,AH,ALNS,ALNH,Z1MAX,Z1DIF,Z2MAX,Z2DIF,
     &                PT,ETAC,ETAD,X1,X2,V,U,QQPD,QQAL,
     &                W,W1,AXX,WEIGHT,MSPR,IREJSC
      COMMON /XSECPT/ SIGS,DSIGHP,SIGH,FS,FH,BETAS(3),AAS,PTCON
C
      CHARACTER*72 TITLE
      DIMENSION BETAB(100)
      DATA IPTSA / 0 /
C
C  selection of pt
      IF(ISOFT.GE.0) THEN
        CALLS = CALLS + ONE
C  sample according to model ISWMDL(3-6)
        IF(ISOFT.GT.1) THEN
 210      CONTINUE
          PTXS = ZERO
          PTYS = ZERO
          DO 300 I=2,ISOFT
            IMODE = ISWMDL(3)
C  valence partons
            IF(IV(I).EQ.1) THEN
              BETA = BETAS(1)
C  photon/Pomeron valence part
              IF(IPAMDL(5).EQ.1) THEN
                IF(XV(I).GE.ZERO) THEN
                  IF((IDPDG1.EQ.22).OR.(IDPDG1.EQ.45)) THEN
                    IMODE = ISWMDL(4)
                    BETA = BETAS(3)
                  ENDIF
                ELSE
                  IF((IDPDG2.EQ.22).OR.(IDPDG2.EQ.45)) THEN
                    IMODE = ISWMDL(4)
                    BETA = BETAS(3)
                  ENDIF
                ENDIF
              ELSE IF(IPAMDL(5).EQ.2) THEN
                BETA = PARMDL(20)
              ELSE IF(IPAMDL(5).EQ.3) THEN
                BETA = BETAS(3)
              ENDIF
C  sea partons
            ELSE IF(IV(I).EQ.0) THEN
              BETA = BETAS(3)
C  hard scattering remnant
            ELSE
              IF(IPAMDL(6).EQ.0) THEN
                BETA = BETAS(1)
              ELSE IF(IPAMDL(6).EQ.1) THEN
                BETA = BETAS(3)
              ELSE
                BETA = PARMDL(20)
              ENDIF
            ENDIF
            BETA = MAX(BETA,0.01D0)
            CALL SELPT(XV(I),ZERO,PTCUT,PTS,BETA,IMODE)
            PTS = MIN(PTMAX,PTS)
            CALL SFECFE(SIG,COG)
            PTSOF(1,I) = COG*PTS
            PTSOF(2,I) = SIG*PTS
            PTXS = PTXS+PTSOF(1,I)
            PTYS = PTYS+PTSOF(2,I)
            BETAB(I) = BETA
 300      CONTINUE
C  balancing of momenta
          PTS = SQRT(PTXS**2+PTYS**2)
          IF(PTS.GE.PTMAX) GOTO 210
          PTSOF(1,1) = -PTXS
          PTSOF(2,1) = -PTYS
          BETAB(1) = 0.D0
C
*400      CONTINUE
C
C  single parton only
        ELSE
          IMODE = ISWMDL(3)
C  valence partons
          IF(IV(1).EQ.1) THEN
            BETA = BETAS(1)
C  photon/Pomeron valence part
            IF(IPAMDL(5).EQ.1) THEN
              IF(XV(1).GE.ZERO) THEN
                IF((IDPDG1.EQ.22).OR.(IDPDG1.EQ.45)) THEN
                  IMODE = ISWMDL(4)
                  BETA = BETAS(3)
                ENDIF
              ELSE
                IF((IDPDG2.EQ.22).OR.(IDPDG2.EQ.45)) THEN
                  IMODE = ISWMDL(4)
                  BETA = BETAS(3)
                ENDIF
              ENDIF
            ELSE IF(IPAMDL(5).EQ.2) THEN
              BETA = PARMDL(20)
            ELSE IF(IPAMDL(5).EQ.3) THEN
              BETA = BETAS(3)
            ENDIF
C  sea partons
          ELSE IF(IV(1).EQ.0) THEN
            BETA = BETAS(3)
C  hard scattering remnant
          ELSE
            IF(IPAMDL(6).EQ.1) THEN
              BETA = BETAS(3)
            ELSE
              BETA = PARMDL(20)
            ENDIF
          ENDIF
          BETA = MAX(BETA,0.01D0)
          CALL SELPT(XV(1),ZERO,PTCUT,PTS,BETA,IMODE)
          PTS = MIN(PTMAX,PTS)
          CALL SFECFE(SIG,COG)
          PTSOF(1,1) = COG*PTS
          PTSOF(2,1) = SIG*PTS
          BETAB(1) = BETA
        ENDIF
C  histogramming
        IF(IPTSA.NE.0) THEN
          DO 500 I=1,ISOFT
            PT = SQRT(PTSOF(1,I)**2+PTSOF(2,I)**2)
            CALL FILHIS(PT,ONE,IPTSA)
 500      CONTINUE
          CALL ADDHIS(IPTSA)
        ENDIF
C  debug output
        IF(IDEB(29).GE.10) THEN
          WRITE(6,'(1X,A,I4)') 'SOFTPT:DEBUG:ISOFT',ISOFT
          WRITE(6,'(6X,A)') 'TABLE OF  I, IV, XV, PT-X, PT-Y, BETA'
          DO 105 I=1,ISOFT
            WRITE(6,'(10X,2I3,4E12.3)') I,IV(I),XV(I),PTSOF(1,I),
     &        PTSOF(2,I),BETAB(I)
 105      CONTINUE
        ENDIF
C
C  initialization of statistics and parameters
C
      ELSE IF(ISOFT.EQ.-1) THEN
        PTSMIN = ZERO
        PTSMAX = PTCUT
C  histogram of pt distribution
        IF((IDEB(29).GE.1).AND.(IPTSA.EQ.0))
     &    CALL NEWHIS(ZERO,5.D0,ZERO,BETAB,30,IPTSA)
        IMODE = -100+ISWMDL(3)
        CALL SELPT(ECMP,PTSMIN,PTSMAX,PTS,BETAS(3),IMODE)
C
C  output of statistics
C
      ELSE IF(ISOFT.EQ.-2) THEN
        IF(IPTSA.NE.0) THEN
          TITLE = 'SOFTPT:DEBUG:PRIMARY SOFT PT DISTRIBUTION '
          CALL OUTHIS(IPTSA,1,TITLE,ONE,1)
          IPTSA = 0
        ENDIF
      ELSE
        WRITE(6,'(1X,A,I2)') 'SOFTPT:ERROR:UNSUPPORTED ISOFT ',ISOFT
        STOP
      ENDIF
      END
