// * This code is licensed under:
// * RunMC License, Version 1.0
// * - for license details see http://runmc.sourceforge.net/License.txt
// *
// * Copyright (c) 2005 by S.Chekanov (chekanov@mail.desy.de).
// * All rights reserved.

#include<iostream>
#include<fstream>
#include<stdlib.h>
#include<math.h>
#include<stdio.h>
#include<string>
#include<unistd.h> // standard unix function for getpid 
#include<sys/types.h>
#include<sys/stat.h>
#include<signal.h>

// for fortran linkage
#include<myroot.h> // for my root files 
#include<TMapFile.h>
// #include<TObjectTable.h> // check memory 

// structure for MC common blocks
#include<heplist.h>
#include<getevkin.h> // put kinematic cuts here  
#include<getseeds.h> // put seeds here 

#ifdef  PYTHIA57
 #include"hepevt_pythia57.h"
#endif

#ifdef  PYTHIA6
  #include"hepevt_pythia62.h"
#endif


#include<hepevt_local.h> // to get local hepevt to write into file
#include<hepevt_runmc.h> // to get RUNMC hepevt to write into file

#include<vector>
#include<time.h>
#include<SetIni.h>    // include class containing initializations for M
#include<SetHisto.h>  // initialization of histograms
using namespace std;


// to access fortran program
extern "C" { 
   void runmcarlo_( int& , int& );   
}


extern void pipeupdate();  
extern Int_t initiate();
extern Int_t dummyrun( int mm );
extern Int_t hepevt_local();         // local hepevt  
extern Int_t hepevt_runmc(Int_t mm); // RunMC hepevt  
extern Int_t getevkin();     // put/get event kinematics  

// internal definitions 
extern Int_t getvalues(char * nnn, double* ); // get all kinematics from the event
extern Int_t getvalues2d(char* nnn1,  char* nnn2, double*, double* ); // get all kinematics from the event (2D) 
extern Int_t getjets( char * nnn, long ncurr, double* ); // get all kinematic for each jet

// fill varibales for histograms
extern Int_t  afill(char * type, char * nnn, Double_t* getval, Long_t nc1=-1, Long_t nc2=-1); 

// analyse HEPEVT 
extern Int_t get_htup( Int_t mode );
// analyse RunMC ntuple
extern Int_t get_rtup( Int_t mode ); 


// define output names of histograms
extern Int_t EventsP;
extern Int_t SingleP;
extern Int_t DoubleP;

// user-defined calculations
extern Int_t user_afill(char * type, char * nnn, Double_t* getval, Long_t nc1=-1, Long_t nc2=-1);
extern Int_t userEvent(char * nnn,   double* );
extern Int_t userInit();
extern Int_t userEnd();
extern Int_t userRun();
extern Int_t userSelect();
extern void PrintList();

extern SetIni inival;
extern SetHisto inihis;
extern hepenvtlocal hlo; // HEPEVT local for output
extern hepenvtrunmc rlo; // HEPEVT RUNMC type for output

heplist hep;       //  fill from HEPEVT and do analysis 
int    CheckWIN32; // check inviroment


// histograms
// for 1D ---------------------------
extern Double_t xmin1D[500];
extern Double_t xmax1D[500];
extern Int_t    nbin1D[500];
extern string   name1D[500];
extern char * NAMESA[500]; // histogram names
extern Int_t EventsP; // event histogram
extern Int_t SingleP; // 1-particle density
extern Int_t DoubleP; // 2-particle density

// for 2D -----------------------------
extern Double_t x1min2D[250];
extern Double_t x2min2D[250];
extern Double_t x1max2D[250];
extern Double_t x2max2D[250];
extern Int_t    n1bin2D[250];
extern Int_t    n2bin2D[250];
extern string   n1name2D[250];
extern string   n2name2D[250];
extern char * NAMESA1[250];
extern char * NAMESA2[250];
extern char * NAMESA3[250]; // histogram names
extern Int_t EventsP1; // event histogram
extern Int_t SingleP1; // 1-particle density
extern Int_t DoubleP1; // 2-particle density
extern Int_t EventsP2; // event histogram
extern Int_t SingleP2; // 1-particle density
extern Int_t DoubleP2; // 2-particle density

// specify order at which histograms are shown
extern Int_t OH[9];

// histograms to fill
// you can assess this
TH1F *hp[500]; 
TH2F *hp2D[250];
char *DrawOption1D[500];
char *DrawOption2D[250];
TFile   *f1=0;   // output file  
TCanvas *c1=0;  // main canvas  


// main program ------------------------------------------------------------ 
int  main(int argc, char **argv) {


// initiation 
    Int_t initi=initiate();
    if (initi == 1) { cout << "error in the initialization, read error file" << endl;
                     };
                      
    Int_t inikin=getevkin(); // set kinematic ranges for MC 
    if (inikin != 0) { cout << "error in initialization of kinematics" << endl; return 1; };

// check inviroment
   CheckWIN32=0;
   if (getenv("RUNMCwin32")  != NULL ) CheckWIN32=1; // seems cygwin 



// get PID of this process, and renice it 
    pid_t mypid = getpid();
    cout << "RUNMC: PID OF THE PROCESS = " <<  mypid << endl;
    if (inival.Renice > 20) {inival.Renice=20;}
    if (inival.Renice < 0)  {inival.Renice=0;}
    char  renicePID[35];
    sprintf(renicePID,"renice %2d %8d", inival.Renice, mypid);
    cout << "RUNMC: "<< renicePID << endl;
    cout << "RUNMC: ";

    Int_t irunrenice=0;
    if (CheckWIN32 == 0) {
    irunrenice=system( renicePID );
    if (irunrenice != 0) {cout << "error in renice" << endl;}; } 


// get seed for MC 
    time_t curtime=time(NULL);      // get current time
    cout << "RUNMC: execution time     = " << ctime(&curtime);


// get seed from time  
    struct tm *tmp = localtime(&curtime);
    extern getseedscommon getseeds_ ;
    getseeds_.RUNMCseed1=10000*((tmp->tm_min)+1)+100*((tmp->tm_sec)+1)+10*(tmp->tm_mday)+(tmp->tm_mon)+tmp->tm_hour;
    getseeds_.RUNMCseed2=10000*(int)mypid+100*((tmp->tm_min)+1)+10*(tmp->tm_mday)+(tmp->tm_sec); 
    cout << "RUNMC: random seeds for this run = " << getseeds_.RUNMCseed1 << " " << getseeds_.RUNMCseed2 << endl;

// infor on stearing cards 
    getseeds_.ISTEER=inival.Steering;


    Int_t  SHOWCANVAS=1; // by deafault,  show canvas  
                         // pop up second window with lumi info 
    if (inihis.HistoNcurrent == 0 || 
        inihis.HistoOptions[1]==5 || 
        inihis.HistoOptions[9]== 0 ) { SHOWCANVAS=0; };

 // in case if you need to access the canvas  
        if (inihis.HistoOptions[1]==6) SHOWCANVAS=2;


    // initiate pipe file 
    char * RUNDIR; 
    char PipeStop[500], PipeEvents[500],MemoryMappedFile[500]; 

    RUNDIR=getenv("RUNMC");
    strcpy(PipeEvents,RUNDIR);
    strcpy(PipeStop,RUNDIR);
    strcpy(MemoryMappedFile,RUNDIR);
//  cout << outpupipes << endl;
    strncat(PipeEvents, "/pipes/", strlen("/pipes/"));
    strncat(PipeEvents, inival.PipeName, strlen(inival.PipeName) );
    strncat(PipeEvents, ".event", strlen(".event") );
    strncat(PipeStop, "/pipes/", strlen("/pipes/"));
    strncat(PipeStop, inival.PipeName, strlen(inival.PipeName) );
    strncat(PipeStop, ".stop", strlen(".stop") );
// make memopy mapped file to hold histograms 
    strncat(MemoryMappedFile,"/pipes/memfile.map", strlen("/pipes/memfile.map") );
    TMapFile::SetMapAddress(0xb46a5000);
    TMapFile* mfile = TMapFile::Create(MemoryMappedFile,"RECREATE", 1000000,
                            "RUNMC histograms mapped in memory");

    // output files 
    char HISTroot[60], HEPEVTroot[60], RUNMCroot[60]; 
    cout << "RUNMC: output file: " <<  inival.MOutput  << endl;
    strncpy(HISTroot, "", 59);
    strncpy(HEPEVTroot, "", 59);
    strncpy(RUNMCroot, "", 59);
    strncat(HISTroot,inival.MOutput, strlen(inival.MOutput));
    strncat(HEPEVTroot,inival.MOutput, strlen(inival.MOutput));
    strncat(RUNMCroot,inival.MOutput, strlen(inival.MOutput));
    strncat(HISTroot, ".root", 5 );
    strncat(HEPEVTroot, ".htup", 5 );
    strncat(RUNMCroot, ".rtup", 5 );


//    remove (HISTroot); // remove old files


// take common block with particle momenta  
// int argc; char **argv;
    Int_t ierr=0,itake; Double_t xgetval1=-999999.0; Double_t xgetval2=-999999.0; 
                        Double_t ygetval1=-999999.0; Double_t ygetval2=-999999.0;
 
// system files
    // open system file for monitoring
    ofstream myout(PipeEvents, ios::out | ios::trunc);
    ifstream myin1(PipeStop, ios::in); //only check at the beginning  
      if (!myout) {
    cout << "Cannot open system file " << PipeEvents << endl; return 1;};
       myout.close();

    if (!myin1) {
    cout << "Cannot open system file! " << PipeStop << endl; 
    cout << "To be able to run without RunMC GUI, you should create this file and put 0" << endl;
    cout << "To stop this run without RunMC GUI, replace 0 by  1" << endl;
    return 1;};
    myin1 >> inival.Stop; myin1.close();
    if (inival.Stop == 1) { cout << PipeStop << " has 1, not 0 : stop this run" << endl; return 2; }

// To access the external COMMON
     extern getevkincommon getevkin_ ;
     extern hepenvtcommon hepevt_ ; 

// root definitions
// gROOT->Reset();
   if (inihis.HistoOptions[5] < 4) { gROOT->SetStyle("Plain"); } 


// inihis.HistoOptions[9]=1 -  histogram files
// inihis.HistoOptions[10]=1 - HEPEVT  files
// inihis.HistoOptions[11]=1 - RunMC   files

// output histograms  
   int  hclocal, myselect; 
   if (inihis.HistoOptions[9] == 1) { f1 = new TFile(HISTroot,"recreate", "RunMC histograms");} 
   Int_t kUPDATE = 5;
   Int_t kEVENTS=100; // how mane events - calculate later  
   if (inihis.HistoOptions[0]==1) kUPDATE = 2; //update interval  
   if (inihis.HistoOptions[0]==2) kUPDATE = 5; //update interval
   if (inihis.HistoOptions[0]==3) kUPDATE = 10; //update interval
   if (inihis.HistoOptions[0]==4) kUPDATE = 30; //update interval
   if (inihis.HistoOptions[0]==5) kUPDATE = 60; //do not upade 


// set new files and branches
  TFile *fhepevt=0; TTree *thepevt=0; TFile *frunmc=0; TTree *trunmc=0; 
  if (inihis.HistoOptions[10] == 1) {
   cout << "RUNMC: write HEPEVT record to file  = " << HEPEVTroot << endl;
   fhepevt = new TFile(HEPEVTroot,"recreate", "HEPEVT output");
   thepevt = new TTree("thepevt","HEPEVT");
   thepevt->Branch("NEVHEP",&hlo.NEVHEP,"NEVHEP/I");
   thepevt->Branch("NHEP",&hlo.NHEP,"NHEP/I");
   thepevt->Branch("ISTHEP",hlo.ISTHEP,"ISTHEP[NHEP]/I");
   thepevt->Branch("IDHEP",hlo.IDHEP,"IDHEP[NHEP]/I");
   thepevt->Branch("JMOHEP1",hlo.JMOHEP1,"JMOHEP1[NHEP]/I");
   thepevt->Branch("JMOHEP2",hlo.JMOHEP2,"JMOHEP2[NHEP]/I");
   thepevt->Branch("JDAHEP1",hlo.JDAHEP1,"JDAHEP1[NHEP]/I");
   thepevt->Branch("JDAHEP2",hlo.JDAHEP2,"JDAHEP2[NHEP]/I");
   thepevt->Branch("CHA3",hlo.CHA3,"CHA3[NHEP]/I"); 
   thepevt->Branch("PX",hlo.PHEP_X,"PX[NHEP]/D");
   thepevt->Branch("PY",hlo.PHEP_Y,"PY[NHEP]/D");
   thepevt->Branch("PZ",hlo.PHEP_Z,"PZ[NHEP]/D");
   thepevt->Branch("E", hlo.PHEP_E,"E[NHEP]/D");
   thepevt->Branch("M", hlo.PHEP_M,"M[NHEP]/D");
   thepevt->Branch("VX",hlo.VHEP_X,"VX[NHEP]/D");
   thepevt->Branch("VY",hlo.VHEP_Y,"VY[NHEP]/D");
   thepevt->Branch("VZ",hlo.VHEP_Z,"VZ[NHEP]/D");
   thepevt->Branch("VT",hlo.VHEP_T,"VT[NHEP]/D"); } 

// reduced form of HEPEVT (RunMC format)  
   if (inihis.HistoOptions[11] == 1) { 
   cout << "RUNMC: write RunMC record to file  = " << RUNMCroot << endl;
   frunmc = new TFile(RUNMCroot,"recreate", "RUNMC output");
   trunmc = new TTree("trunmc","RunMC");
   trunmc->Branch("NHEP",&rlo.NHEP,"NHEP/I");
   trunmc->Branch("IDHEP",rlo.IDHEP,"IDHEP[NHEP]/I");
   trunmc->Branch("ISTCHA",rlo.ISTCHA,"ISTCHA[NHEP]/I");
   trunmc->Branch("JMOHEP",rlo.JMOHEP,"JMOHEP[NHEP]/I");
   trunmc->Branch("JDAHEP",rlo.JDAHEP,"JDAHEP[NHEP]/I");
   trunmc->Branch("PX",rlo.PHEP_X,"PX[NHEP]/F");
   trunmc->Branch("PY",rlo.PHEP_Y,"PY[NHEP]/F");
   trunmc->Branch("PZ",rlo.PHEP_Z,"PZ[NHEP]/F");
   trunmc->Branch("E", rlo.PHEP_E,"E[NHEP]/F");
   }


    TApplication* runmcApp=0;
   // run canvas
    if (SHOWCANVAS>0)  runmcApp = new TApplication("App", &argc, argv);

// show canvas application
   if (inihis.HistoOptions[9] == 1) {f1->cd(); } // back to histogram 
   Int_t xcan=inihis.HistoOptions[3];
   Int_t ycan=inihis.HistoOptions[4];
   Int_t  nshow=8; // show all histograms (up to 8 histograms for now) 

//------------------- start showing canvas -----------------------------------------------
   if (SHOWCANVAS > 0 ) {  // if show canves   
   c1 = new TCanvas("c1", "RunMC ROOT",xcan,ycan);
   c1->SetLineWidth(2);
   c1->SetTitle( "RunMC"  );  // put title
   if (inihis.HistoOptions[6]==2) {c1->SetLogx();}
   if (inihis.HistoOptions[7]==2) {c1->SetLogy();}

// some color options
   if (inihis.HistoOptions[5] < 4)  { c1->SetFillColor(0); };
   if (inihis.HistoOptions[5]==6) { // color  style markers
   c1->SetFrameFillColor(41);
   c1->SetGrid(); c1->SetLineWidth(2); c1->SetLineColor(kMagenta); c1->SetFillColor(41); } 


 if (inihis.HistoOptions[1]==1) {
// this make log scale to work for one histogram
//     if (inihis.HistoNcurrent == 1) {
//     c1->Divide(1,1,0.005,0.005); }
     if (inihis.HistoNcurrent == 2) { 
     c1->Divide(1,2,0.005,0.005); }
     if (inihis.HistoNcurrent == 3) { 
     c1->Divide(2,2,0.005,0.005); }
     if (inihis.HistoNcurrent == 4) { 
     c1->Divide(2,2,0.005,0.005); }
     if (inihis.HistoNcurrent == 5) { 
     c1->Divide(2,3,0.005,0.005); }
     if (inihis.HistoNcurrent == 6) { 
     c1->Divide(2,3,0.005,0.005); }
     if (inihis.HistoNcurrent == 7) { 
     c1->Divide(2,4,0.005,0.005); }
     if (inihis.HistoNcurrent >= 8) { 
     c1->Divide(2,4,0.005,0.005); }
   } 
   c1->Draw();

// define maximum number of shown histograms
   if (inihis.HistoOptions[1]==2) {
     nshow=1;  
     c1->Divide(1,1,0.005,0.005); }
   if (inihis.HistoOptions[1]==3) {
     nshow=2;
     c1->Divide(1,2,0.005,0.005); }
   if (inihis.HistoOptions[1]==4) {
     nshow=4; 
     c1->Divide(2,2,0.005,0.005); }
   if (inihis.HistoOptions[1]==5) {
     nshow=0; }        

   gPad->SetLeftMargin(0.2);
   gPad->SetBottomMargin(0.25);
   if (inihis.HistoOptions[6]==2) {gPad->SetLogx();}
   if (inihis.HistoOptions[7]==2) {gPad->SetLogy();}
   c1->Update();

   } // end show canvas  defined 

// ------------------- end showing canvas 
// different styles
    char hmode[20];
    char hmode2D[20];
    char * OOP1="hist e";
    char * OOP2="ep";
    char * OOPD[10]={"LEGO","LEGO2","LEGO","SURF","SURF2Z","COL","COLZ","CONT0","ARR","BOX"};

   Float_t xmin=0; Float_t xmax=0; Int_t nbin=0;
   Float_t bin_size[500];
  
   if (inihis.HistoNcurrent >= 500) { cout << "Too many histograms!" << endl; return 3;}

   
   cout << "RunMC: the following order was set to show the histograms= "<< endl;
   cout << "       "<< OH[1] << "D " << OH[2] << "D " << OH[3] << "D " << OH[4] << "D ";
   cout << OH[5] << "D " << OH[6] << "D " << OH[7] << "D " << OH[8] << "D " << endl;
   cout << "RUNMC: total number of 1D histograms                 =" << inihis.HistoNcurrent1D << endl;
   cout << "       number of 1D histograms for events            =" << EventsP << endl;
   cout << "       number of 1D histograms for 1-particle density=" << SingleP << endl;
   cout << "       number of 1D histograms for 2-particle density=" << DoubleP << endl;
// create 1D histograms
   for  (Int_t i=1; i<=inihis.HistoNcurrent1D; i++ ) {
    cout <<"RUNMC: title of 1D histogram=" << i << "  " << NAMESA[i] << endl;
    xmin=xmin1D[i];
    xmax=xmax1D[i];
    nbin=nbin1D[i];
    bin_size[i]=(xmax-xmin)/(Float_t)nbin;
   hp[i]  = new TH1F(NAMESA[i],NAMESA[i],nbin,xmin,xmax);
   hp[i]->SetTitle( NAMESA[i] );    
   hp[i]->SetLineStyle(1);
   hp[i]->SetFillColor(0);
   hp[i]->SetLineWidth(2);
   hp[i]->SetMarkerStyle(20);
   hp[i]->SetMarkerSize(0.3);
   hp[i]->Sumw2();
   if (inihis.HistoOptions[5]==1) { // color style lines
   hp[i]->SetFillColor(45);
   strcpy(hmode,OOP1);  }; 
   if (inihis.HistoOptions[5]==2) { // plain  style lines
   hp[i]->SetFillColor(0);
   strcpy(hmode,OOP1); };
   if (inihis.HistoOptions[5]==3) { // plain  style markers 
   hp[i]->SetFillColor(0);   
   hp[i]->SetMarkerStyle(20); hp[i]->SetMarkerSize(1.0);  strcpy(hmode,OOP2); };
   if (inihis.HistoOptions[5]==4) { // color  style markers
   hp[i]->SetFillColor(0); hp[i]->SetMarkerStyle(20); hp[i]->SetMarkerSize(1.0); 
   strcpy(hmode,OOP2);; };
   if (inihis.HistoOptions[5]==5) { // color  style markers
   hp[i]->SetFillColor(33); hp[i]->SetMarkerStyle(20); hp[i]->SetMarkerSize(1.0); 
   hp[i]->SetMarkerColor(kBlue); 
   hp[i]->SetMarkerStyle(20); strcpy(hmode,OOP2); };
   if (inihis.HistoOptions[5]==6) { // color  style markers
   hp[i]->SetFillColor(kBlue);
   strcpy(hmode,OOP1);  
   };
   mfile->Add(hp[i],NAMESA[i]); // add to memory mapped file  

// set drawing options. This can be changed latter by a user
   DrawOption1D[i] = new char[strlen(hmode)];
   strcpy(DrawOption1D[i],hmode);
// set statistics
   if (inihis.HistoOptions[2]==1) {hp[i]->SetStats(1);}
   if (inihis.HistoOptions[2]==2) {hp[i]->SetStats(0);}
   }

  cout << "RUNMC: number of 2D histograms                       =" << inihis.HistoNcurrent2D << endl;
// create 2D histograms
  for  (Int_t i=1; i<=inihis.HistoNcurrent2D; i++ ) {
  cout <<"RUNMC: title of 2D histogram=" << i << "  " << NAMESA3[i] << " build from ";  
  cout << NAMESA1[i] << " and " << NAMESA2[i] <<  endl;
  cout << "       number of 2D histograms for events            =" << EventsP1 << " and " << EventsP2 << endl;
  cout << "       number of 2D histograms for 1-particle density=" << SingleP1 << " and " << SingleP2 << endl;
  cout << "       number of 2D histograms for 2-particle density=" << DoubleP1 << " and " << DoubleP2 << endl;
  hp2D[i]  = new TH2F(NAMESA3[i],NAMESA3[i],n1bin2D[i],x1min2D[i],x1max2D[i],n2bin2D[i],x2min2D[i],x2max2D[i]);
  hp2D[i]->SetTitle( NAMESA[i] );
  hp2D[i]->SetLineStyle(1);
  hp2D[i]->SetFillColor(0);
  hp2D[i]->SetLineWidth(2);
  hp2D[i]->SetMarkerStyle(20);
  hp2D[i]->SetMarkerSize(0.3);
  hp2D[i]->SetTitle( NAMESA3[i] );
  hp2D[i]->Sumw2();
  mfile->Add(hp2D[i],NAMESA3[i]); // add to memory mapped file

// draw options - user can change this 
  DrawOption2D[i] = new char[strlen(OOPD[inihis.HistoOptions[12]])];
  strcpy(DrawOption2D[i],OOPD[inihis.HistoOptions[12]]);
  }

// get 2D styles - only for RunMC  
   strcpy(hmode2D,OOPD[inihis.HistoOptions[12]]);



   Int_t n=0; int nev=0; ierr=1; Int_t check1=0; Int_t check2=0; 
                                 Int_t check3=0; Int_t check4=0; 
                                

// user initialisation
    Int_t userinierror=userInit();
    if (userinierror !=0 ) { cout << "userInit() returns error!" << endl;  return 2; }


// initialize Monte Carlo
    if (inival.MCtype > 0) {

// user initialisation
    cout << "RUNMC: Total number of events to generate = " << inival.Nevents  << endl;

// MC initialisation
    itake=1; runmcarlo_(itake, ierr);
    cout << "RUNMC: initialize Monte Carlo, call itake=" << itake << endl; 


// now ntuples 
     } else if ( inival.MCtype == -1 ) { 
          check3=get_rtup( -1 ); // initialize RunMC ntuple 
          if (check3 != 0) {cout << "Error in get_rtup().." << endl;   return 1; } 
          cout << "RUNMC: Total number of events in RTUP ntuple = " << inival.Nevents  << endl;
// now ntuples
     } else if ( inival.MCtype == -2 ) {
          check3=get_htup( -1 ); // initialize ntuple HEPEVET
          if (check3 != 0) {cout << "Error in get_htup().." << endl;   return 1; }
          cout << "RUNMC: Total number of events in HTUP ntuple = " << inival.Nevents  << endl;
        }


 

// to deal with names
  string NEWP1,NEWP2;
  string a("@");

// make timer for average time per event 
    TStopwatch RUNMCtimer;
    RUNMCtimer.Start();
    Double_t RUNMCrealTime=0;
    Double_t RUNMCcpuTime=0;
    Int_t    EventsFor5sec=1;
    Int_t    NTimeCheck=50000; // check average time after this number 
    Int_t    NtimeC=0;

//   Create a timer object to benchmark this loop
   TStopwatch timer;
   timer.Start(); Int_t itime=0; Double_t tnew = 0;


// run over MC ---------------------------------------------------- 
   while(n<inival.Nevents) {
   n++; 
  
// terminate  
   if (inival.Stop == 1) {
   cout << "RUNMC: Terminate the job, since inival.Stop is" << inival.Stop << endl;
   goto stopend;} 

// do first MC 
   if (inival.MCtype > 0) {itake=2; runmcarlo_(itake, ierr); };       //fill HEPEVT 
   if (inival.MCtype == -1 ) check3=get_rtup( n-1 ); // get events from RUNMC ntuple  
   if (inival.MCtype == -2 ) check3=get_htup( n-1 ); // get events from HEPEVT ntuple  


   if (hepevt_.NHEP >= 0) {                 // take if the event looks OK  
 
   nev++; // count events which look OK 


   if (n == 0 || (n >= NTimeCheck && (n%NTimeCheck) == 0)  ) {
   NtimeC=0;  RUNMCtimer.Start(); }  
   NtimeC++;
// check average time  
   if (NtimeC == 20) {
   RUNMCtimer.Stop();
   RUNMCrealTime = RUNMCtimer.RealTime()/ 20. ;
   RUNMCcpuTime = timer.CpuTime() / 20. ;
     cout << "RUNMC: average CPU  time for 1 event = " << RUNMCcpuTime;
     cout << " after " <<  n << " events" << endl;
     cout << "RUNMC: average REAL time for 1 event = " << RUNMCrealTime;
     cout << " after " <<  n << " events" << endl; 
     EventsFor5sec=(Int_t)(5./RUNMCrealTime);
     kEVENTS=(Int_t)( (Float_t)kUPDATE / RUNMCrealTime);
     //  cout << "No of events necessary for 5 sec " << EventsFor5sec << endl;
     if (3*EventsFor5sec < 2) {
        cout << "Looks like your program or computer is too slow!" << cout;
        cout << "RUNMC takes " << RUNMCrealTime << " to process a single event!" << cout;
        cout << "RUNMC will try to terminate the job " << endl;
        goto stopend; };
    };


// ouput
        if (n<=10 &&
        n<=100 && (n%10) == 0 ||
        n<=1000 && (n%100) == 0  ||
        n>=1000 && (n%1000) == 0. ) {
        tnew = timer.RealTime();
        timer.Continue();
        itime = (Int_t)tnew;
        cout << "No events= " << n << " time passed=" << itime << " s" << endl; };

// output to system file: number of events and stop signal
        if (
        (n<=100 && (n%10) == 0) ||
        (n>100 && (n%EventsFor5sec) == 0)    ) { 
// check termination
        ifstream myin(PipeStop, ios::in);
        if (!myin) { 
            cout << "Cannot open system pipe file: " << PipeStop << endl;
            cout << "Since it was removed, RunMC assumes that you want to terminate this run"  << endl; 
            cout << "All histograms should be saved!" << endl; 
            goto stopend; 
        } else { myin >> inival.Stop; } myin.close(); 

 
// write into
        ofstream myout(PipeEvents, ios::out | ios::trunc);
        myout << n << " " << mypid << endl;   myout.close();  }  // put number of events and pid number

// fill final state 
      if (inival.FStype == 1) { hep.fill_ALLstable();         // fill stable particles  
      } else if (inival.FStype == 2)  {hep.fill_Partons();           // fill partons 
      } else if (inival.FStype == 3)  {hep.fill_All();               // fill all particles 
      } else if (inival.FStype == 4)  {hep.fill_CHARGEDstable();     // fill charged particles
      } else if (inival.FStype == 5)  {hep.fill_HEPEVT(); hep.fill_ALLstable(); 
      } else if (inival.FStype == 6)  { // user defined - do not do anything 
      } else { cout << "Wrong definition of the final state!" << endl; return 1; }

// --------------------- here you can fill trees ------------------------------------------
// fill TTree with HEPEVT
// no any event selection is assumed here
   if (inihis.HistoOptions[10] == 1) {
      hclocal=hepevt_local();
   if (hclocal == 0) {fhepevt->cd(); thepevt->Fill(); }  }


// fill TTree with RunMC HEPEVT
// event selection is assumed here: all stable, all charged, partons
// in addition, you can preselect events bu the user subroutine
// if you want event, myselect=0
   if (inihis.HistoOptions[11] == 1) {
      hclocal=hepevt_runmc( inival.FStype );
      myselect=userSelect();
   if (hclocal == 0 && myselect == 0) {frunmc->cd(); trunmc->Fill(); }  }

//--------------------------------------------------------------------------------------------


// do sothing with the list of particles
    if (inival.FStype != 6 ) {  // not for user defined final state 
      if (inival.JetType == 1) { } // do not do anything  
      if (inival.JetType == 2) hep.set_JADEjets();         // do JADE calculation
      if (inival.JetType == 3) hep.set_DURHAMjets();       // do DURHAM 
      if (inival.JetType == 4) hep.set_KTjets();           // do KT long. invar

//move to frame (JETS)  
      if (inival.Frame == 1)  { } // lab frame 

      if (inival.JetType == 1) { 
      if (inival.Frame == 2) hep.set_particleLABtoHCMframe();         // move to hadronic CM
      if (inival.Frame == 3) hep.set_particleLABtoBREITframe();       // move to Breit frame
      } else {
      if (inival.Frame == 2) hep.set_jetLABtoHCMframe();         // move to hadronic CM 
      if (inival.Frame == 3) hep.set_jetLABtoBREITframe();       // move to Breit frame
      }; 

// print event
//  PrintList();

//if particles are selected, set cuts
      if (inival.JetType == 1) {hep.set_particlePtEta(); 
      } else {                  hep.set_jetPtEta();    }; // for jets 

     } // end non-user definitions 
 
// print event 
// PrintList(); 


// user run over events  
   Int_t userrunerror=userRun();
   if (userrunerror !=0 ) { cout << "userRun() returns error!" << endl;  goto stopend; }



//  fill histograms for event characteristics 
   if (inihis.HistoOptions[9] == 1) { // start filling histograms  
// user selection 
// take event if it returns 0
   Int_t mget=userSelect();


// back to histogram files
     f1->cd(); 


     if (inival.FStype != 6 ) {  // not for user defined final state

// fill 1D histograms for events 
      if  (EventsP != 0) { 
      for  (Int_t i=1; i<=inihis.HistoNcurrent1D; i++ ) {
        string nf1 = name1D[i].substr(0,1);
          if ( a != nf1) { // run if no "@" found 
          check1 = afill( "particle", NAMESA[i], &xgetval1 );
          check2 = user_afill( "particle", NAMESA[i], &xgetval2 );
          if (check1==0 && mget ==0) hp[i]->Fill( xgetval1, (Stat_t)inihis.HistoWeight[i] ); 
          if (check2==0 && mget ==0) hp[i]->Fill( xgetval2, (Stat_t)inihis.HistoWeight[i] ); // user calculations 
       };
       }; // end for 
       }; 


// fill 2D histogram for event varibales
      if  (EventsP1 != 0 && EventsP2 != 0 ) {
      for  (Int_t i=1; i<=inihis.HistoNcurrent2D; i++ ) {
       string nf1=n1name2D[i].substr(0,1); 
       string nf2=n2name2D[i].substr(0,1);
       if ( a != nf1 && a != nf2 ) { // run if no "@" found
          check1 = afill( "particle", NAMESA1[i], &xgetval1);
          check2 = afill( "particle", NAMESA2[i], &ygetval1);
          if (check1==0 && check2==0 && mget ==0) {
          hp2D[i]->Fill( xgetval1, ygetval1, (Stat_t)inihis.HistoWeight[i] ); };
          check1 = user_afill( "particle", NAMESA1[i], &xgetval2);
          check2 = user_afill( "particle", NAMESA2[i], &ygetval2);
          if (check1==0 && check2==0 && mget ==0) {
          hp2D[i]->Fill( xgetval2, ygetval2, (Stat_t)inihis.HistoWeight[i] ); };
       }; }; }; 

// fill 2D histogram : 1-particle density + 1-particle density for jets
     if (inival.JetType > 1) {
     if  (SingleP1 != 0 &&  SingleP2 !=0 && DoubleP1 ==0 && DoubleP2==0) {
       for  (Int_t i=1; i<=inihis.HistoNcurrent2D; i++ ) {
      string nf1a=n1name2D[i].substr(0,1); 
      string nf2a=n1name2D[i].substr(1,1);
      string nf1b=n2name2D[i].substr(0,1);
      string nf2b=n2name2D[i].substr(1,1);
     if ( a == nf1a && a != nf2a ) { // run if "@" found
     if ( a == nf1b && a != nf2b ) { // run if "@" found
     for  (unsigned int nn=0; nn<hep.PJ.size(); nn++ ) {
     for  (unsigned int mm=0; mm<hep.PP.size(); mm++ ) {

      check1 = afill( "particle", NAMESA1[i], &xgetval1, nn);
      check2 = afill( "jet",      NAMESA2[i], &ygetval1, mm);
      if (check1==0 && check2==0 && mget ==0) {
       hp2D[i]->Fill( xgetval1, ygetval1, (Stat_t)inihis.HistoWeight[i] ); };

// invert
      check1 = afill( "jet", NAMESA1[i], &xgetval1, nn);
      check2 = afill( "particle", NAMESA2[i], &ygetval1, mm);
      if (check1==0 && check2==0 && mget ==0) {
       hp2D[i]->Fill( xgetval1, ygetval1, (Stat_t)inihis.HistoWeight[i] ); };


      check1 = user_afill( "particle", NAMESA1[i], &xgetval1, nn);
      check2 = user_afill( "jet",      NAMESA2[i], &ygetval1, mm);
      if (check1==0 && check2==0 && mget ==0) {
       hp2D[i]->Fill( xgetval1, ygetval1, (Stat_t)inihis.HistoWeight[i] ); };
 
// invert
      check1 = user_afill( "jet", NAMESA1[i], &xgetval1, nn);
      check2 = user_afill( "particle", NAMESA2[i], &ygetval1, mm);
      if (check1==0 && check2==0 && mget ==0) {
       hp2D[i]->Fill( xgetval1, ygetval1, (Stat_t)inihis.HistoWeight[i] ); };

     }; }; }; };
     }; }; };  // endif




// loop over particles or jets -------------------------------------------------------
      Int_t Iruns=1;
      char * stype[2]={"particle","jet"};
      unsigned int  entries[2];
      entries[0]=hep.PP.size();
      entries[1]=0; 
      if (inival.JetType > 1) { Iruns=2; entries[1]=hep.PJ.size();}
      for  (Int_t GetType=0; GetType<Iruns; GetType++ ) {
 
 
// fill histgrams for each particles (single-particle density) 
      if  (SingleP != 0) {
        for  (Int_t i=1; i<=inihis.HistoNcurrent1D; i++ ) {
        string nf1 = name1D[i].substr(0,1);
        string nf2 = name1D[i].substr(1,1);
        if ( a == nf1 && a != nf2 ) { // run if "@" found
           for  (unsigned int nn=0; nn<entries[GetType]; nn++ ) { 
           check1 = afill(stype[GetType], NAMESA[i], &xgetval1, nn );
           check2 = user_afill( stype[GetType], NAMESA[i], &xgetval2, nn );
           if (check1==0 && mget ==0) hp[i]->Fill( xgetval1, (Stat_t)inihis.HistoWeight[i]  ); 
           if (check2==0 && mget ==0) hp[i]->Fill( xgetval2, (Stat_t)inihis.HistoWeight[i]  );
       }  // end loop
       } // end if
       } // end loop over all particles  
       } // end if


// fill histgrams for particle pairs (two-particle density) 
      if (entries[GetType] > 1) {  
      if  (DoubleP != 0) {
        for  (Int_t i=1; i<=inihis.HistoNcurrent1D; i++ ) {
        string nf1 = name1D[i].substr(0,1);
        string nf2 = name1D[i].substr(1,1);
        if ( a == nf1 && a == nf2 ) { // run if "@@" found
           for  (unsigned int nn=0; nn<entries[GetType]-1; nn++ ) {
           for  (unsigned int mm=nn+1; mm<entries[GetType]; mm++ ) {
           check1 = afill( stype[GetType], NAMESA[i], &xgetval1, nn, mm );
           check2 = user_afill( stype[GetType], NAMESA[i], &xgetval2, nn, mm );
           if (check1==2 || check2==2) {cout << "Error in fill function" << endl; return 1;} 
           if (check1==0 && mget ==0) hp[i]->Fill( xgetval1, (Stat_t)inihis.HistoWeight[i]  );
           if (check2==0 && mget ==0) hp[i]->Fill( xgetval2, (Stat_t)inihis.HistoWeight[i]  ); 
       }; };
       };   // end if
       };    // end loop over all particles
       }; }; // end if



// fill 2D histograms----------------------------------------
// fill 2D histgrams for each particles (single-particle density)
      if  (SingleP1 != 0 || SingleP2 !=0 && DoubleP1 ==0 && DoubleP2==0) {
        for  (Int_t i=1; i<=inihis.HistoNcurrent2D; i++ ) {
      string nf1a=n1name2D[i].substr(0,1);
      string nf2a=n1name2D[i].substr(1,1);
      string nf1b=n2name2D[i].substr(0,1);
      string nf2b=n2name2D[i].substr(1,1);
//     cout << stype[GetType] << endl;
// only single-particle densities 
     if ( a == nf1a && a != nf2a ) { // run if "@" found
     if ( a == nf1b && a != nf2b ) { // run if "@" found
     for  (unsigned int nn=0; nn<entries[GetType]; nn++ ) {
       check1 = afill( stype[GetType], NAMESA1[i], &xgetval1, nn);
       check2 = afill( stype[GetType], NAMESA2[i], &ygetval1, nn);
 
   //    cout << stype[GetType] << " " << NAMESA1[i] << " " << NAMESA2[i] << endl;

      if (check1==0 && check2==0 && mget ==0) {
       hp2D[i]->Fill( xgetval1, ygetval1, (Stat_t)inihis.HistoWeight[i] ); };

       check1 = user_afill( stype[GetType], NAMESA1[i], &xgetval2, nn);
       check2 = user_afill( stype[GetType], NAMESA2[i], &ygetval2, nn);
       if (check1==0 && check2==0 && mget ==0) {
       hp2D[i]->Fill( xgetval2, ygetval2, (Stat_t)inihis.HistoWeight[i] ); };
     } // end loop 
     };   };   // end if

// single-particle + events: firsr Event variable, second - single-particle
    if ( (a != nf1a && a != nf2a) && (a == nf1b && a != nf2b) ) { 
       check1 = afill( stype[GetType], NAMESA1[i], &xgetval1);
       for  (unsigned int nn=0; nn<entries[GetType]; nn++ ) {
       check2 = afill( stype[GetType], NAMESA2[i], &ygetval1, nn);
       if (check1==0 && check2==0 && mget ==0) hp2D[i]->Fill( xgetval1, ygetval1, (Stat_t)inihis.HistoWeight[i] );
       };
// for user
       check1 = user_afill( stype[GetType], NAMESA1[i], &xgetval2);
       for  (unsigned int nn=0; nn<entries[GetType]; nn++ ) {
       check2 = user_afill( stype[GetType], NAMESA2[i], &ygetval2, nn);
       if (check1==0 && check2==0 && mget ==0) hp2D[i]->Fill( xgetval2, ygetval2, (Stat_t)inihis.HistoWeight[i] ); 
       };
     } // endif 

// single-particle + events: firsr single-particle, second - event 
    if ( (a == nf1a && a != nf2a) && (a != nf1b && a != nf2b) ) {
       check1 = afill( stype[GetType], NAMESA2[i], &ygetval1);
       for  (unsigned int nn=0; nn<entries[GetType]; nn++ ) {
       check2 = afill( stype[GetType], NAMESA1[i], &xgetval1, nn);
       if (check1==0 && check2==0 && mget ==0) hp2D[i]->Fill( xgetval1, ygetval1, (Stat_t)inihis.HistoWeight[i] );
       };
// for user
       check1 = user_afill( stype[GetType], NAMESA2[i], &ygetval2);
       for  (unsigned int nn=0; nn<entries[GetType]; nn++ ) {
       check2 = user_afill( stype[GetType], NAMESA1[i], &xgetval2, nn);
       if (check1==0 && check2==0 && mget ==0) hp2D[i]->Fill( xgetval2, ygetval2, (Stat_t)inihis.HistoWeight[i] );
       };
     } // endif


     } // end loop over histograms 
     } // end if


// now fill 2D histograms if at least one varibale is 2-particle density
      if  (DoubleP1 !=0 ||  DoubleP2 !=0) {
        for  (Int_t i=1; i<=inihis.HistoNcurrent2D; i++ ) {
      string nf1a=n1name2D[i].substr(0,1);
      string nf2a=n1name2D[i].substr(1,1);
      string nf1b=n2name2D[i].substr(0,1);
      string nf2b=n2name2D[i].substr(1,1);
// event & 2-particle density; first event, second 2-particles 
         if ( (a != nf1a && a != nf2a)  && (a == nf1b && a== nf2b) )  {
         check1 = afill( stype[GetType], NAMESA1[i], &xgetval1);
         check2 = user_afill( stype[GetType], NAMESA1[i], &xgetval2); };

         if ( a == nf1b && a == nf2b ) { // run if "@@" found
           for  (unsigned int nn=0; nn<entries[GetType]-1; nn++ ) {
           for  (unsigned int mm=nn+1; mm<entries[GetType]; mm++ ) {
           check3 = afill( stype[GetType], NAMESA2[i], &ygetval1, nn, mm );
           check4 = user_afill( stype[GetType], NAMESA2[i], &ygetval2, nn, mm );
           if (check1==0 && check3 ==0 && mget ==0) hp2D[i]->Fill( xgetval1, ygetval1, (Stat_t)inihis.HistoWeight[i] ); 
           if (check2==0 && check4 ==0 && mget ==0) hp2D[i]->Fill( xgetval2, ygetval2, (Stat_t)inihis.HistoWeight[i] );;
       }; }; }; 

// deal with the inverse situation: first 2-particle, second event
         if ( (a != nf1b && a != nf2b) && (a == nf1a && a== nf2a)  )  {
         check1 = afill( stype[GetType], NAMESA2[i], &ygetval1);
         check2 = user_afill( stype[GetType], NAMESA2[i], &ygetval2); };
 
         if ( a == nf1a && a == nf2a ) { // run if "@@" found
           for  (unsigned int nn=0; nn<entries[GetType]-1; nn++ ) {
           for  (unsigned int mm=nn+1; mm<entries[GetType]; mm++ ) {
           check3 = afill( stype[GetType], NAMESA1[i], &xgetval1, nn, mm );
           check4 = user_afill( stype[GetType], NAMESA1[i], &xgetval2, nn, mm );
           if (check1==0 && check3 ==0 && mget ==0) hp2D[i]->Fill( xgetval1, ygetval1, (Stat_t)inihis.HistoWeight[i] );
           if (check2==0 && check4 ==0 && mget ==0) hp2D[i]->Fill( xgetval2, ygetval2, (Stat_t)inihis.HistoWeight[i] );;
       }; }; };


// now deal with 2-particle vs 1-particle densities
// first variable single-particle, second -2-particl
       if ( (a == nf1a && a != nf2a) && (a == nf1b && a== nf2b) )  {
       for  (unsigned int nn1=0; nn1<entries[GetType]; nn1++ ) {
       check1 = afill( stype[GetType], NAMESA1[i], &xgetval1, nn1);
       check2 = user_afill( stype[GetType], NAMESA1[i], &xgetval2, nn1);
           for  (unsigned int nn=0; nn<entries[GetType]-1; nn++ ) {
           for  (unsigned int mm=nn+1; mm<entries[GetType]; mm++ ) {
           check3 = afill( stype[GetType], NAMESA2[i], &ygetval1, nn, mm );
           check4 = user_afill( stype[GetType], NAMESA2[i], &ygetval2, nn, mm );
           if (check1==0 && check3 ==0 && mget ==0) hp2D[i]->Fill( xgetval1, ygetval1, (Stat_t)inihis.HistoWeight[i] );
           if (check2==0 && check4 ==0 && mget ==0) hp2D[i]->Fill( xgetval2, ygetval2, (Stat_t)inihis.HistoWeight[i] );;
       }; }; // end of 2-particle density  
       }; // end single-particle loop 
       }; // end if   

// opposite situation - first 2-particle, second -single particle 
       if ( (a == nf1b && a != nf2b) && (a == nf1a && a== nf2a) )  {
       for  (unsigned int nn1=0; nn1<entries[GetType]; nn1++ ) {
       check1 = afill( stype[GetType], NAMESA2[i], &ygetval1, nn1);
       check2 = user_afill( stype[GetType], NAMESA2[i], &ygetval2, nn1);
           for  (unsigned int nn=0; nn<entries[GetType]-1; nn++ ) {
           for  (unsigned int mm=nn+1; mm<entries[GetType]; mm++ ) {
           check3 = afill( stype[GetType], NAMESA1[i], &xgetval1, nn, mm );
           check4 = user_afill( stype[GetType], NAMESA1[i], &xgetval2, nn, mm );
           if (check1==0 && check3 ==0 && mget ==0) hp2D[i]->Fill( xgetval1, ygetval1, (Stat_t)inihis.HistoWeight[i] );
           if (check2==0 && check4 ==0 && mget ==0) hp2D[i]->Fill( xgetval2, ygetval2, (Stat_t)inihis.HistoWeight[i] );;
       }; }; // end of 2-particle density
       }; // end single-particle loop
       }; // end if



       }; }; 


       };
// end loop over particles and jets

      }; // end non-user defined state 

// update memory mapped file
      if ( n && (n%kEVENTS) == 0 )  mfile->Update();

// start updatting 
      if ( n && (n%kEVENTS) == 0 && SHOWCANVAS > 0  ) {
           Int_t ncoun1=0; Int_t ncoun2=0;
           for  (Int_t i=1; i<=inihis.HistoNcurrent; i++ ) {
           if (i<=nshow) { 
           c1->cd( i );
           if (OH[i] == 1) { ncoun1++;    hp[ncoun1]->Draw( DrawOption1D[ncoun1] );  };
           if (OH[i] == 2) { ncoun2++;  hp2D[ncoun2]->Draw( DrawOption2D[ncoun2] );  }; 
           c1->Update();
           }; }; 
       if (gSystem->ProcessEvents()) break;  
       }  // end show canvas  defined 


       } // End filling of histograms ------------------------------------------------------



   }; // end if statement when looping over good events 
   } // end loop over events  

 stopend:

// finish event 
// terminate MC 
     if (inival.MCtype > 0) {
        itake=3; runmcarlo_(itake, ierr);
        cout << " All done by analmc, call RUNMCARLO with itake=" << itake << endl;
     }
// write final number of processed events 
     ofstream myout1(PipeEvents, ios::out | ios::trunc);
     myout1 << n << endl;    // put number of events
     myout1.close();

     Float_t Snormalize, Lumi=0; // normalization factor and lumi 
     if (inival.MCtype > 0) Lumi=(Float_t)nev / getevkin_.xCross;
     if (inival.MCtype < 0) Lumi=inival.NTlumi;
     cout << " Int. Lumi=" << Lumi << " pb-1  for " << nev << " MC events"  << endl;


// if this is 2D histogram, put axis (only for single histogram!)
      if (inihis.HistoNcurrent2D == 1 && inihis.HistoNcurrent1D == 0) {
         c1->cd( 1 ); 
         TAxis *axis1 = hp2D[1]->GetXaxis();
         TAxis *axis2 = hp2D[1]->GetYaxis(); 
         axis1->SetTitleSize(0.07); axis2->SetTitleSize(0.07);
         axis1->SetTitleOffset(1.0);
         axis1->SetTitle( NAMESA1[1]  ); 
         axis2->SetTitle( NAMESA2[1]  );
         axis1->Draw("same");
         axis2->Draw("same");
         if (SHOWCANVAS > 0) c1->Update();
       } // end axis



// normalize 1D histograms ---------------------------------------------------
       for  (Int_t i=1; i<=inihis.HistoNcurrent1D; i++ ) {
// normalize to 1 all histograms
       if (inihis.HistoOptions[8]==2) {
        Snormalize=hp[i]->Integral();
        cout << i << " histogram is normalized to 1. The integral is=" << Snormalize << endl;
        hp[i]->Scale( 1./ Snormalize );
       }
// get differential cross sections in pb
       if (inihis.HistoOptions[8]==3) {
       Snormalize= Lumi * bin_size[i] ;
       cout << i << " histogram is normalized to get cross section (in pb): Normalisation=" << Snormalize << endl;
       hp[i]->Scale( 1./ Snormalize );
       }
// get differential cross sections in nb 
       if (inihis.HistoOptions[8]==4) {
       Snormalize= 1000.* Lumi * bin_size[i] ;
       cout << i << " histogram is normalized to get cross section (in nb): Normalisation=" << Snormalize << endl;
       hp[i]->Scale( 1./ Snormalize );
       }
       } // end run over 1D
// normalize 1D histograms --------------------------------------------------
       for  (Int_t i=1; i<=inihis.HistoNcurrent2D; i++ ) {
// normalize to 1 all histograms
       if (inihis.HistoOptions[8]==2) {
        Snormalize=hp2D[i]->Integral();
        cout << i << " 2D histogram is normalized to 1. The integral is=" << Snormalize << endl;
        hp2D[i]->Scale( 1./ Snormalize );
       }
       } // no cross section normalisations for 2D histograms !


// replot again all histograms 
         Int_t ncoun1=0; Int_t ncoun2=0;
         for  (Int_t i=1; i<=inihis.HistoNcurrent; i++ ) {
         if (i<=nshow && SHOWCANVAS > 0) {
         c1->cd( i );
         if (OH[i] == 1) { ncoun1++;    hp[ncoun1]->Draw( DrawOption1D[ncoun1] );  };
         if (OH[i] == 2) { ncoun2++;  hp2D[ncoun2]->Draw( DrawOption2D[ncoun2] );  };
         c1->Update();
         }; };
 
// if this is 2D histogram, put axis (only for single histogram!)
      if (inihis.HistoNcurrent2D == 1 && inihis.HistoNcurrent1D == 0) {
         c1->cd( 1 );
         TAxis *axis1 = hp2D[1]->GetXaxis();
         TAxis *axis2 = hp2D[1]->GetYaxis();
         axis1->SetTitleSize(0.07); axis2->SetTitleSize(0.07);
         axis1->SetTitleOffset(1.0);
         axis1->SetTitle( NAMESA1[1]  );
         axis2->SetTitle( NAMESA2[1]  );
         axis1->Draw("same");
         axis2->Draw("same");
         if (SHOWCANVAS > 0 && inihis.HistoOptions[8]==1) c1->Update();
       } // end axis



// user termination
   Int_t userenderror=userEnd();
   if (userenderror !=0 ) { cout << "userEnd() returns error!" << endl;  return 3; }
 
// start App 
    if (SHOWCANVAS>0) runmcApp->Run(kTRUE);

// stop timer
    timer.Stop();


// write histograms into file and close 
   if (inihis.HistoOptions[9] == 1) { 
   f1->cd(); f1->Write(); f1->Close(); } // close file 
   
// close HEPEVT output file
   if (inihis.HistoOptions[10] == 1) {
   cout << "----------------------------------------------"  << endl;
   cout << "RUNMC: closing HEPEVT record with file name  = " << HEPEVTroot << endl;
   cout << "-----------------------------------------------" << endl;
   fhepevt->cd(); thepevt->Print(); thepevt->Write(); } // close file 

// close RunMC format 
   if (inihis.HistoOptions[11] == 1) { 
   cout << "----------------------------------------------"  << endl;
   cout << "RUNMC: closing RunMC record with file name  = " << RUNMCroot << endl;
   cout << "-----------------------------------------------" << endl;
   frunmc->cd(); trunmc->Print(); trunmc->Write();  } // close file


// fill output file 
     char OutputFinal[500]; 
     strcpy(OutputFinal,RUNDIR);    
     strncat(OutputFinal, "/proj/.runmc.out", strlen("/proj/.runmc.out"));
     ofstream finalOut(OutputFinal, ios::out);

     if (finalOut) {
     char  definestuff[200], mcinfo[100];
     finalOut << "# RunMC 4.0 output onformation" << endl; 
     if (inihis.HistoOptions[8]==1) strcpy(definestuff,"Unnormalized  histograms");
     if (inihis.HistoOptions[8]==2) strcpy(definestuff,"Normalized to 1 histograms");
     if (inihis.HistoOptions[8]==3) strcpy(definestuff,"Differential x-section in pb");
     if (inihis.HistoOptions[8]==4) strcpy(definestuff,"Differential x-section in nb");
// output information
     if (inival.MCtype==1) {strcpy(mcinfo,"PYTHIA 6.3");
     } else if (inival.MCtype==2) {strcpy(mcinfo,"HERWIG 6.5");
     } else if (inival.MCtype==3) {strcpy(mcinfo,"ARIADNE 4.12");
     } else if (inival.MCtype==4) {strcpy(mcinfo,"LEPTO 6.5l");
     } else if (inival.MCtype==5) {strcpy(mcinfo,"AROMA 2.2");
     } else if (inival.MCtype==6) {strcpy(mcinfo,"CASCADE 1.2");
     } else if (inival.MCtype==7) {strcpy(mcinfo,"PHOJET 1.05");
     } else if (inival.MCtype==8) {strcpy(mcinfo,"RAPGAP 3.0");
     } else if (inival.MCtype==9) {strcpy(mcinfo,"ThePEG 1.0");
     } else if (inival.MCtype==-1) {strcpy(mcinfo,"RunMC ntuple");
     } else if (inival.MCtype==-2) {strcpy(mcinfo,"HEPEVT ntuple");
     } else { strcpy(mcinfo,"undefined model"); };
 
      char     uevents1[40], uevents2[40];
      sprintf( uevents1, "%d", n );
      sprintf( uevents2, "%d", nev );
      char     xsect[40];
      sprintf( xsect, "%e pb", getevkin_.xCross );
      char     xlumi[40];
      sprintf( xlumi, "%e pb-1", Lumi );

      finalOut << "monte-carlo-model= " << mcinfo << endl;
      finalOut << "output= " << definestuff << endl;
      finalOut << "events-requested= " <<  uevents1 << endl;
      finalOut << "events-generated= " <<  uevents2 << endl;
      finalOut << "total-cross-section= " <<  xsect << endl;
      finalOut << "luminosity= " <<  xlumi << endl;


     // close HEPEVT output file
   if (inihis.HistoOptions[10] == 1) {
      finalOut << "output-hepevt-record= " << HEPEVTroot << endl; 
    }  

    if (inihis.HistoOptions[11] == 1) {
     finalOut << "output-root-record= " << RUNMCroot << endl;
    }

      finalOut << "average-CPU-time-for-1-event= " << RUNMCcpuTime << endl;
      finalOut << "average-REAL-time-for-1-event= " << RUNMCrealTime <<  endl;

       time_t curtime2=time(NULL);      // get current time
       char    time1[50], time2[50];
       sprintf( time1, "%s", ctime(&curtime) );
       sprintf( time2, "%s", ctime(&curtime2) );

       finalOut << "run-started= " <<  time1; 
       finalOut << "run-finished= " <<  time2;


     } else {
      cout << "----------------------------------------------"  << endl;
      cout << "Error: cannot write $RUNMC/proj/.proj.out file" << endl;
      cout << "----------------------------------------------" << endl;
     }  // end show info canvas

    finalOut.close();

// close mapped file
    mfile->Close();

// clean temporary files if there are some 
     struct stat buf;
     int itmp = stat ("runmc.tmp",&buf);
     if (itmp == 0) remove("runmc.tmp");
 
   //  we print the list of objects known to ROOT
   //
   // if (gObjectTable) gObjectTable->Print();

 
    return 0;
}
